@@ -34,16 +34,35 @@
 #include <sys/times.h>
 #include <gtk/gtk.h>
 #include <errno.h>
+#include <locale.h>
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#include "measurements.h"
+
+// Define GSourceOnceFunc if compiling against an older version of GLIB:
+#if GLIB_VERSION_CUR_STABLE < G_ENCODE_VERSION(2,74)
+typedef void (* GSourceOnceFunc) (gpointer user_data);
+#endif
+
 #undef _Complex_I
 #define _Complex_I  (__extension__ 1.0i)
 #undef I
 #define I _Complex_I
 
+/* GNU/Hurd has no hard PATH_MAX */
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
+// Use __LOCATION__ anywhere you want the file and line number:
+// (Move this to be used in all calls to mem_realloc)
+#define __S1(x) #x
+#define __S2(x) __S1(x)
+#define  __LOCATION__ __FILE__ ":"  __S2(__LINE__)
+
 /*
  * Standard gettext macros.
  */
@@ -236,12 +255,13 @@ typedef struct
   /* Current working directory */
   char working_dir[FILENAME_LEN];
 
-  /* xnec2c's glade file */
-  char xnec2c_glade[FILENAME_LEN];
-
   /* Current NEC2 input file */
   char input_file[FILENAME_LEN];
 
+  /* If set true, then use GTK loops for the frequency loop iteration
+   * instead of spawning a pthread */
+  int disable_pthread_freqloop;
+
   /* Main (structure) window position and size */
   int
     main_x,
@@ -315,8 +335,41 @@ typedef struct
   /* Preferred mathlib index, if available */
   int mathlib_idx, mathlib_batch_idx;
 
+  /* See enum GAIN_SCALE */
+  int gain_style;
+
+  /* If true, then call Fit_to_Scale() on the X-axis
+   * fr_plot->min_fscale/fr_plot->max_fscale values. */
+  int freqplots_round_x_axis;
+
+  /* if true, exit after the first frequency loop iteration */
+  int batch_mode;
+
+  /* verbose and debug levels, see console.h */
+  int verbose, debug;
+
+  /* true if ~/.xnec2c/xnec2c.conf does not exist, false otherwise */
+  int first_run;
+
+  /* optimization file write flags */
+  int opt_write_csv;
+  int opt_write_s1p;
+  int opt_write_s2p_max_gain;
+  int opt_write_s2p_viewer_gain;
+
+  /* If for example --write-csv <file> is added to the cmdline then write
+   * the file after each frequency iteration */
+  char *filename_csv;
+  char *filename_s1p;
+  char *filename_s2p_max_gain;
+  char *filename_s2p_viewer_gain;
 } rc_config_t;
 
+typedef struct {
+	void (*callback)(char *);
+	char *extension;
+} filechooser_t;
+
 /* Gain Scaling style */
 enum GAIN_SCALE
 {
@@ -510,7 +563,8 @@ typedef struct
     ksymp,  /* Ground flag */
     ifar,   /* Int flag in RP card, for far field calculations */
     iperf,  /* Type of ground flag */
-    nradl;  /* Number of radials in ground screen */
+    nradl,  /* Number of radials in ground screen */
+    gpflag; /* The gpflag from the GE card */
 
   double
     t2,     /* Const for radial wire ground impedance */
@@ -741,6 +795,10 @@ typedef struct {
 	// Pointer to &calc_data.freq_loop_data[fr]
 	freq_loop_data_t *freq_loop_data;
 
+	// Scaled versions of fr_plot->freq_loop_data->min_freq/max_freq for plot display
+	// for use with rc_config.freqplots_round_x_axis
+	double min_fscale, max_fscale;
+
 	// Because we are using realloc it is hard to know if the structure has
 	// been initialized or if it needs to be set to sane values.  The 
 	// value will equal 0xc2bca3083893e65e (just a 64-bit random number) if
@@ -819,8 +877,7 @@ typedef struct
     *max_gain_tht,  /* Theta angle where maximum gain occurs */
     *max_gain_phi,  /*   Phi angle where maximum gain occurs */
     *tilt,          /* Tilt angle of polarization ellipse  */
-    *axrt,          /* Elliptic axial ratio of pol ellipse */
-    fbratio;        /* Front to Back gain ratio */
+    *axrt;          /* Elliptic axial ratio of pol ellipse */
 
   int
     *max_gain_idx,  /* Where in rad_pattern.gtot the max value occurs */
@@ -1289,6 +1346,7 @@ void Set_Window_Labels(void);
 void Alloc_Rdpattern_Buffers(int nfrq, int nth, int nph);
 void Alloc_Nearfield_Buffers(int n1, int n2, int n3);
 void Free_Draw_Buffers(void);
+double Scale_Gain( double gain, int fstep, int idx );
 /* draw_structure.c */
 void Draw_Structure(cairo_t *cr);
 void New_Patch_Data(void);
@@ -1339,6 +1397,10 @@ gboolean patch(int nx, int ny, double ax1, double ay1, double az1, double ax2, d
 gboolean reflc(int ix, int iy, int iz, int iti, int nop);
 void wire(double xw1, double yw1, double zw1, double xw2, double yw2, double zw2, double rad, double rdel, double rrad, int ns, int itg);
 /* gnuplot.c */
+void Save_FreqPlots_S1P(char *filename);
+void Save_FreqPlots_S2P_Max_Gain(char *filename);
+void Save_FreqPlots_S2P_Viewer_Gain(char *filename);
+void Save_FreqPlots_CSV(char *filename);
 void Save_FreqPlots_Gnuplot_Data(char *filename);
 void Save_RadPattern_Gnuplot_Data(char *filename);
 void Save_Struct_Gnuplot_Data(char *filename);
@@ -1410,6 +1472,7 @@ void load(int *ldtyp, int *ldtag, int *ldtagf, int *ldtagt, double *zlr, double
 /* optimize.c */
 void Write_Optimizer_Data(void);
 void *Optimizer_Output(void *arg);
+int opt_have_files_to_save();
 /* plot_freqdata.c */
 void Plot_Frequency_Data(cairo_t *cr);
 void Plots_Window_Killed(void);
@@ -1435,6 +1498,8 @@ gboolean Nec2_Save_Warn(const gchar *mesg);
 int Load_Line(char *buff, FILE *pfile);
 void mem_alloc(void **ptr, size_t req, gchar *str);
 void mem_realloc(void **ptr, size_t req, gchar *str);
+void mem_backtrace(void *ptr);
+void mem_obj_dump(void *ptr);
 void free_ptr(void **ptr);
 gboolean Open_File(FILE **fp, char *fname, const char *mode);
 void Close_File(FILE **fp);
@@ -1448,11 +1513,14 @@ void SaveFlag(unsigned long long int *flag, unsigned long long int mask);
 void Strlcpy(char *dest, const char *src, size_t n);
 void Strlcat(char *dest, const char *src, size_t n);
 double Strtod(char *nptr, char **endptr);
+char *str_append(char *dst, char *a, char *b, size_t n);
 void Get_Dirname(char *fpath, char *dirname, int *fname_idx);
 void xnec2_widget_queue_draw(GtkWidget *w);
-void g_idle_add_once(GSourceFunc function, gpointer data);
-void g_idle_add_once_sync(GSourceFunc function, gpointer data);
-void print_backtrace(void);
+#if GLIB_VERSION_CUR_STABLE < G_ENCODE_VERSION(2,74)
+guint g_idle_add_once(GSourceOnceFunc function, gpointer data);
+#endif
+guint g_idle_add_once_sync(GSourceOnceFunc function, gpointer data);
+void print_backtrace(char *msg);
 
 /* xnec2c.c */
 void Near_Field_Pattern(void);
@@ -1463,5 +1531,6 @@ gboolean Frequency_Loop(gpointer udata);
 gboolean Start_Frequency_Loop(void);
 void Stop_Frequency_Loop(void);
 void Incident_Field_Loop(void);
+int set_freq_step();
 
 #endif
