@@ -21,13 +21,95 @@
 #include "shared.h"
 #include "mathlib.h"
 
+#include <getopt.h>
+
 static void sig_handler(int signal);
 
 /* Child process pid returned by fork() */
 static pid_t child_pid = (pid_t)(-1);
 
+enum XNEC2C_OPTS {
+	// Start at 128 after all single-digit opts:
+	OPT_FIRST_OPT = 128,
+
+	OPT_ENABLE_OPTIMIZE,
+
+	OPT_WRITE_CSV,
+	OPT_WRITE_S1P,
+	OPT_WRITE_S2P_MAX_GAIN,
+	OPT_WRITE_S2P_VIEWER_GAIN,
+
+	OPT_MAX_OPTS
+};
+
+static struct option long_options[] = {
+		{  "input",                  required_argument,   NULL,  'i'                        },
+		{  "jobs",                   required_argument,   NULL,  'j'                        },
+		{  "help",                   no_argument,         NULL,  'h'                        },
+		{  "verbose",                no_argument,         NULL,  'v'                        },
+		{  "debug",                  no_argument,         NULL,  'd'                        },
+		{  "quiet",                  no_argument,         NULL,  'q'                        },
+		{  "version",                no_argument,         NULL,  'V'                        },
+		{  "no-pthreads",            no_argument,         NULL,  'P'                        },
+		{  "batch",                  no_argument,         NULL,  'b'                        },
+
+		{  "optimize",               no_argument,         NULL,  OPT_ENABLE_OPTIMIZE        },
+
+		{  "write-csv",              required_argument,   NULL,  OPT_WRITE_CSV              },
+		{  "write-s1p",              required_argument,   NULL,  OPT_WRITE_S1P              },
+		{  "write-s2p-max-gain",     required_argument,   NULL,  OPT_WRITE_S2P_MAX_GAIN     },
+		{  "write-s2p-viewer-gain",  required_argument,   NULL,  OPT_WRITE_S2P_VIEWER_GAIN  },
+
+		{  NULL,                     0,                   NULL,  0                          }
+	};
+
+static char *build_optstring(struct option *long_options)
+{
+	static char optstring[256] = {0};
+	char c[2] = {0,0};
+
+	int i;
+	for (i = 0; long_options[i].name != NULL; i++)
+	{
+		if (long_options[i].val < OPT_FIRST_OPT)
+			c[0] = (char)long_options[i].val;
+		else
+			continue;
+
+		strcat(optstring, c);
+
+		if (long_options[i].has_arg == required_argument)
+			strcat(optstring, ":");
+		else if (long_options[i].has_arg == optional_argument)
+			strcat(optstring, "::");
+	}
+
+	return optstring;
+}
+
+static gint opt_start_optimizer_thread()
+{
+	GtkWidget *w;
+
+	w = Builder_Get_Object(main_window_builder, "main_freqplots");
+	if (!gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM(w)))
+	{
+		Notice(_("Xnec2c Optimize"),
+			_("Frequency plots window is not configured. "
+			"Select your desired graphs and click the triangle \"Play\" button."), GTK_BUTTONS_OK);
+		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(w), TRUE);
+	}
+
+	w = Builder_Get_Object(main_window_builder, "optimizer_output");
+	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(w), TRUE);
+
+	return FALSE;
+}
+
 /*------------------------------------------------------------------------*/
 
+char *orig_numeric_locale = NULL;
+
   int
 main (int argc, char *argv[])
 {
@@ -39,11 +121,22 @@ main (int argc, char *argv[])
   /* new and old actions for sigaction() */
   struct sigaction sa_new, sa_old;
 
+  // Print all notices that may occur before getopt parsing:
+  rc_config.verbose = 9;
+
   /* initialize new actions */
   sa_new.sa_handler = sig_handler;
   sigemptyset( &sa_new.sa_mask );
   sa_new.sa_flags = 0;
 
+  // Setup locales so we can switch between C and the system locale.
+  // The pointer returned from setlocale() seems to be stack sensitive
+  // so make a copy of it for later:
+  setlocale(LC_ALL, "");
+  char *l = setlocale(LC_NUMERIC, NULL);
+  mem_alloc((void**)&orig_numeric_locale, strlen(l)+1, __LOCATION__);
+  strcpy(orig_numeric_locale, l);
+
   /* Register function to handle signals */
   sigaction( SIGINT,  &sa_new, &sa_old );
   sigaction( SIGSEGV, &sa_new, NULL );
@@ -54,72 +147,18 @@ main (int argc, char *argv[])
 
   gtk_init (&argc, &argv);
 
-  /* Create the file path to xnec2c Glade file */
-  Strlcpy( rc_config.xnec2c_glade,
-      getenv("HOME"), sizeof(rc_config.xnec2c_glade) );
-  Strlcat( rc_config.xnec2c_glade,
-      "/.xnec2c/xnec2c.glade", sizeof(rc_config.xnec2c_glade) );
-
-  /* Check for the Glade config file */
-  FILE *fp = fopen( rc_config.xnec2c_glade, "r" );
-  if( fp == NULL )
-  {
-    fprintf( stderr, "xnec2c: cannot open xnec2c Glade GUI Description file.\n" );
-    perror( rc_config.xnec2c_glade );
-    fprintf( stderr, "xnec2c: trying to create xnec2c config directory "
-                     "from the installation prefix file tree.\n" );
-
-    /* Find the binary's path (location in file system) */
-    char exe_path[256], file_path[288];
-
-    /* Read the file path to xnec2c executable */
-    size_t len = sizeof( exe_path );
-    int bytes = (int)readlink( "/proc/self/exe", exe_path, len );
-    if( bytes <= 0 )
-    {
-      fprintf( stderr, "xnec2c: cannot read xnec2c binary's location.\n" );
-      perror( "/proc/self/exe" );
-      exit( -1 );
-    }
-
-    /* Remove "/bin/xnec2c" from the path with room for termination */
-    bytes -= sizeof( "/bin/xnec2c" ) - 1;
-    if( bytes < 1 )
-    {
-      fprintf( stderr, "xnec2c: cannot create file path to examples/xnec2c.\n" );
-      exit( -1 );
-    }
-
-    /* Create file path to xnec2c examples directory */
-    exe_path[bytes] = '\0';
-    Strlcpy( file_path, exe_path, sizeof(file_path) );
-    Strlcat( file_path, "/share/examples/xnec2c", sizeof(file_path) );
-    fprintf( stderr, "xnec2c: creating xnec2c config directory from: %s\n", file_path );
-
-    /* Create system command to copy examples/xnec2c to ~/.xnec2c */
-    char syscmnd[512];
-    Strlcpy( syscmnd, "cp -r ", sizeof(syscmnd) );
-    Strlcat( syscmnd, file_path, sizeof(syscmnd) );
-    Strlcat( syscmnd, " ", sizeof(syscmnd) );
-    Strlcat( syscmnd, getenv("HOME"), sizeof(syscmnd) );
-    Strlcat( syscmnd, "/.xnec2c",   sizeof(syscmnd) );
-    int ret = system( syscmnd );
-    if( ret == -1 )
-    {
-      fprintf( stderr,"xnec2c: cannot create xnec2c's working directory.\n" );
-      perror( file_path );
-      exit( -1 );
-    }
-  } /* if( fp == NULL ) */
-  else fclose( fp );
-
   /* Create a default config if needed, abort on error */
   if( !Create_Default_Config() ) exit( -1 );
 
   /* Process command line options */
   calc_data.num_jobs  = 1;
   rc_config.input_file[0] = '\0';
-  while( (option = getopt(argc, argv, "i:j:hv") ) != -1 )
+
+  // default to show warnings or more important errors.
+  rc_config.verbose = 4; 
+
+  int option_index = 0;
+  while( (option = getopt_long(argc, argv, build_optstring(long_options), long_options, &option_index) ) != -1 )
   {
     switch( option )
     {
@@ -128,8 +167,7 @@ main (int argc, char *argv[])
           size_t siz = sizeof( rc_config.input_file );
           if( strlen(optarg) >= siz )
           {
-            fprintf ( stderr,
-                _("xnec2c: input file name too long ( > %d char )\n"), (int)siz - 1 );
+            pr_crit("input file name too long ( > %d char )\n", (int)siz - 1);
             exit(-1);
           }
           /* For null term. */
@@ -137,49 +175,119 @@ main (int argc, char *argv[])
         }
         break;
 
+      case 'v': /* increase verbosity */
+        rc_config.verbose++;
+        break;
+
+      case 'd': /* debug */
+        rc_config.debug++;
+        break;
+
+      case 'q': /* quiet */
+        rc_config.debug = 0;
+        rc_config.verbose = 0;
+        break;
+
       case 'j': /* number of child processes = num of processors */
         calc_data.num_jobs = atoi( optarg );
 
         if (calc_data.num_jobs == 0)
         {
-            printf("[%d] Forking disabled!\n", getpid());
+            pr_notice("Forking disabled!\n");
             enable_forking = 0;
             calc_data.num_jobs = 1;
         }
         break;
 
+      case 'P': /* disable pthread loop */
+        rc_config.disable_pthread_freqloop = 1;
+        pr_notice("pthread freqloop disabled!\n");
+        break;
+
+      case 'b': /* batch mode */
+        pr_notice("batch mode enabled, will exit after first loop\n");
+        rc_config.batch_mode = 1;
+        rc_config.main_loop_start = 1;
+        break;
+
       case 'h': /* print usage and exit */
         usage();
         exit(0);
+        break;
 
-      case 'v': /* print xnec2c version */
+      case 'V': /* print xnec2c version */
         puts( PACKAGE_STRING );
         exit(0);
+        break;
+
+      case OPT_ENABLE_OPTIMIZE:
+          SetFlag( OPTIMIZER_OUTPUT );
+          break;
+
+      case OPT_WRITE_CSV:
+        rc_config.filename_csv = optarg;
+        break;
+
+      case OPT_WRITE_S1P:
+        rc_config.filename_s1p = optarg;
+        break;
+
+      case OPT_WRITE_S2P_MAX_GAIN:
+        rc_config.filename_s2p_max_gain = optarg;
+        break;
+
+      case OPT_WRITE_S2P_VIEWER_GAIN:
+        rc_config.filename_s2p_viewer_gain = optarg;
+        break;
 
       default:
         usage();
         exit(0);
+        break;
 
     } /* switch( option ) */
   } /* while( (option = getopt(argc, argv, "i:o:hv") ) != -1 ) */
 
+  if (rc_config.batch_mode && isFlagSet(OPTIMIZER_OUTPUT))
+  {
+	  pr_crit("--batch and --optimize are mutual exclusive.\n");
+	  exit(1);
+  }
+
   /* Initialize the external math libraries */
   init_mathlib();
 
   /* Read input file path name if not supplied by -i option */
-  if( (strlen(rc_config.input_file) == 0) &&
-      (strstr(argv[argc - 1], ".nec") ||
-       strstr(argv[argc - 1], ".NEC")) )
+  while (strlen(rc_config.input_file) == 0 && optind < argc)
   {
-    size_t siz = sizeof( rc_config.input_file );
-    if( strlen(argv[argc - 1]) >= siz )
+    if (strstr(argv[optind], ".nec") || strstr(argv[optind], ".NEC"))
     {
-      fprintf ( stderr,
-          _("xnec2c: input file path name too long ( > %d char )\n"), (int)siz - 1 );
-      exit(-1);
+      size_t siz = sizeof(rc_config.input_file);
+
+      if (strlen(argv[optind]) >= siz)
+      {
+        pr_crit("input file path name too long ( > %d char )\n", (int) siz - 1);
+        exit(1);
+      }
+
+      Strlcpy(rc_config.input_file, argv[optind], siz);
     }
-     /* For null termination */
-    Strlcpy( rc_config.input_file, argv[argc-1], siz );
+    else
+      pr_warn("unexpected argument '%s' does not appear to be a .nec file\n", argv[optind]);
+
+    optind++;
+  }
+
+  while (optind < argc)
+  {
+    pr_warn("unexpected argument: %s\n", argv[optind]);
+    optind++;
+  }
+
+  if (strlen(rc_config.input_file) == 0 && rc_config.batch_mode)
+  {
+	  pr_crit("batch mode requires an input file\n");
+	  exit(1);
   }
 
   /* When forking is useful, e.g. if more than 1 processor is
@@ -206,18 +314,16 @@ main (int argc, char *argv[])
       err = pipe( forked_proc_data[idx]->pnt2child_pipe );
       if( err )
       {
-        perror( "xnec2c: pipe()" );
-        fprintf( stderr,
-            _("xnec2c: exiting after fatal error: pipe() failed") );
+        perror( "pipe()" );
+        pr_crit("exiting after fatal error: pipe() failed");
         exit(-1);
       }
 
       err = pipe( forked_proc_data[idx]->child2pnt_pipe );
       if( err )
       {
-        perror( "xnec2c: pipe()" );
-        fprintf( stderr,
-            _("xnec2c: exiting after fatal error: pipe() failed") );
+        perror( "pipe()" );
+        pr_crit("exiting after fatal error: pipe() failed");
         exit(-1);
       }
 
@@ -225,9 +331,8 @@ main (int argc, char *argv[])
       forked_proc_data[idx]->child_pid = fork();
       if( forked_proc_data[idx]->child_pid == -1 )
       {
-        perror( "xnec2c: fork()" );
-        fprintf( stderr,
-            _("xnec2c: exiting after fatal error: fork() failed") );
+        perror( "fork()" );
+        pr_crit("exiting after fatal error: fork() failed");
         exit(-1);
       }
       else child_pid = forked_proc_data[idx]->child_pid;
@@ -325,8 +430,13 @@ main (int argc, char *argv[])
 
   init_mathlib_menu();
 
+  if (isFlagSet(OPTIMIZER_OUTPUT))
+	  g_idle_add_once((GSourceOnceFunc)opt_start_optimizer_thread, NULL);
+
   gtk_main ();
 
+  free_ptr((void**)&orig_numeric_locale);
+
   return 0;
 } // main()
 
@@ -486,7 +596,6 @@ Open_Input_File( gpointer arg )
     /* Simulate activation of main rdpattern button */
     if( isFlagClear(OPTIMIZER_OUTPUT) )
       Main_Rdpattern_Activate( FALSE );
-    crnt.valid = 0;
 
     /* Select display of radiation or EH pattern */
     if( isFlagSet(DRAW_GAIN) )
@@ -565,23 +674,23 @@ static void sig_handler( int signal )
   switch( signal )
   {
     case SIGINT:
-      fprintf( stderr, _("xnec2c: exiting via user interrupt\n") );
+      if (!CHILD) pr_crit("exiting via user interrupt\n");
       break;
 
     case SIGSEGV:
-      fprintf( stderr, _("xnec2c: segmentation fault, exiting\n") );
+      pr_crit("segmentation fault, exiting\n");
       break;
 
     case SIGFPE:
-      fprintf( stderr, _("xnec2c: floating point exception, exiting\n") );
+      pr_crit("floating point exception, exiting\n");
       break;
 
     case SIGABRT:
-      fprintf( stderr, _("xnec2c: abort signal received, exiting\n") );
+      pr_crit("abort signal received, exiting\n");
       break;
 
     case SIGTERM:
-      fprintf( stderr, _("xnec2c: termination request received, exiting\n") );
+      pr_crit("termination request received, exiting\n");
       break;
 
     case SIGCHLD:
@@ -591,8 +700,7 @@ static void sig_handler( int signal )
 
         if( !FORKED )
         {
-          fprintf( stderr,
-              _("xnec2c: not forked, ignoring SIGCHLD from pid %d\n"), pid );
+          pr_crit("not forked, ignoring SIGCHLD from pid %d\n", pid);
           return;
         }
         else
@@ -600,7 +708,7 @@ static void sig_handler( int signal )
           for( idx = 0; idx < calc_data.num_jobs; idx++ )
             if( forked_proc_data[idx]->child_pid == pid )
             {
-              fprintf( stderr, _("xnec2c: child process pid %d exited\n"), pid );
+              pr_crit("child process pid %d exited\n", pid);
               if( isFlagSet(MAIN_QUIT) ) return;
               else break;
             }
@@ -609,7 +717,7 @@ static void sig_handler( int signal )
       }
 
     default:
-      fprintf( stderr, _("xnec2c: default exit with signal: %d\n"), signal );
+      pr_debug("default exit with signal: %d\n", signal);
   } /* switch( signal ) */
 
   /* Kill child processes */
