@@ -39,15 +39,14 @@ static mathlib_t mathlibs[] = {
 	{.type = MATHLIB_ATLAS, .lib = "libtatlas.so.3", .name = "ATLAS, Threaded", .f_prefix = "clapack_"},
 	{.type = MATHLIB_ATLAS, .lib = "libsatlas.so.3", .name = "ATLAS, Serial", .f_prefix = "clapack_"},
 
-	// CentOS 7: yum install openblas-devel
-	{.type = MATHLIB_OPENBLAS, .lib = "libopenblas.so",  .name = "OpenBLAS+LAPACKe, Serial", .f_prefix = "LAPACKE_"},
-	{.type = MATHLIB_OPENBLAS, .lib = "libopenblaso.so", .name = "OpenBLAS+LAPACKe, OpenMP", .f_prefix = "LAPACKE_"},
-	{.type = MATHLIB_OPENBLAS, .lib = "libopenblasp.so", .name = "OpenBLAS+LAPACKe, pthreads", .f_prefix = "LAPACKE_"},
-
+	// CentOS 7: yum install openblas-devel # or openblas-{serial,threads,openmp}
 	// openSUSE OpenBLAS: zypper install libopenblas_*0
-	{.type = MATHLIB_OPENBLAS, .lib = "libopenblas_serial.so.0",  .name = "OpenBLAS+LAPACKe, Serial", .f_prefix = "LAPACKE_"},
-	{.type = MATHLIB_OPENBLAS, .lib = "libopenblas_openmp.so.0", .name = "OpenBLAS+LAPACKe, OpenMP", .f_prefix = "LAPACKE_"},
-	{.type = MATHLIB_OPENBLAS, .lib = "libopenblas_pthreads.so.0", .name = "OpenBLAS+LAPACKe, pthreads", .f_prefix = "LAPACKE_"},
+	{.type = MATHLIB_OPENBLAS, .lib = "libopenblas.so.0,libopenblas.so,libopenblas_serial.so.0",
+		.name = "OpenBLAS+LAPACKe, Serial", .f_prefix = "LAPACKE_"},
+	{.type = MATHLIB_OPENBLAS, .lib = "libopenblaso.so.0,libopenblaso.so,libopenblas_openmp.so.0",
+		.name = "OpenBLAS+LAPACKe, OpenMP", .f_prefix = "LAPACKE_"},
+	{.type = MATHLIB_OPENBLAS, .lib = "libopenblasp.so.0,libopenblasp.so,libopenblas_pthreads.so.0",
+		.name = "OpenBLAS+LAPACKe, pthreads", .f_prefix = "LAPACKE_"},
 
 	// Ubuntu / Debian: apt-get install liblapacke libopenblas0-*
 	// Tested Ubuntu 16.04, 18.04, 20.04, Debian 9, Debian 11
@@ -125,12 +124,13 @@ void close_mathlib(mathlib_t *lib)
 
 int open_mathlib(mathlib_t *lib)
 {
+	char *libfn0 = NULL, *libfn, *token;
 	char fname[40];
 	int fidx;
 
 	if (lib == NULL)
 	{
-		printf("open_mathlib: lib is NULL\n");
+		BUG("open_mathlib: lib is NULL\n");
 		return 0;
 	}
 
@@ -140,24 +140,37 @@ int open_mathlib(mathlib_t *lib)
 
 	// Set environment if configured:
 	if (lib->env[0] != NULL && setenv(lib->env[0], lib->env[1], 1) < 0)
-			printf("setenv(%s, %s): %s\n", lib->env[0], lib->env[1], strerror(errno));
+			pr_debug("setenv(%s, %s): %s\n", lib->env[0], lib->env[1], strerror(errno));
 
 	// Clear any error state
 	dlerror();
 
-	// Open the .so library
-	lib->handle = dlopen(lib->lib, RTLD_NOW);
+	// Open the .so library, split on a comma (,):
+	mem_alloc((void**)&libfn0, strlen(lib->lib)+1, __LOCATION__);
+	strcpy(libfn0, lib->lib);
+	libfn = libfn0;
+	while ((token = strtok_r(libfn, ",", &libfn)) != NULL)
+	{
+		lib->handle = dlopen(token, RTLD_NOW);
+		if (lib->handle == NULL)
+		{
+			pr_info("%s: %s\n", lib->name, dlerror());
+			close_mathlib(lib);
+		}
+		else
+		{
+			pr_info("%s: opened %s\n", lib->name, token);
+			break;
+		}
+	}
+	free_ptr((void**)&libfn0);
 
 	if (lib->handle == NULL)
 	{
-		printf("  Unable to open %s: %s\n", lib->lib, dlerror());
-		close_mathlib(lib);
+		pr_info("%s: library not found or nonfunctional, skipping: %s\n", lib->name, lib->lib);
 		return 0;
 	}
 
-	if (dlinfo(lib->handle, RTLD_DI_LMID, &lib->lmid) == -1)
-		printf("dlinfo: %s: %s\n", lib->lib, dlerror());
-
 	// Call the init() function if configured
 	if (lib->init != NULL)
 		lib->init(lib);
@@ -179,7 +192,7 @@ int open_mathlib(mathlib_t *lib)
 
 		if (error != NULL)
 		{
-			printf("  %s: unable to bind %s: %s\n", lib->lib, mathfuncs[fidx], error);
+			pr_err("  %s: unable to bind %s: %s\n", lib->lib, mathfuncs[fidx], error);
 			close_mathlib(lib);
 			break;
 		}
@@ -203,11 +216,8 @@ void init_mathlib()
 		mathlibs[libidx].functions = NULL;
 
 		// Try to open each library:
-		printf("\nTrying %s (%s):\n", mathlibs[libidx].name, mathlibs[libidx].lib);
 		if (!open_mathlib(&mathlibs[libidx]))
 		{
-			printf("  skipping.\n");
-
 			close_mathlib(&mathlibs[libidx]);
 
 			mathlibs[libidx].available = 0;
@@ -217,14 +227,7 @@ void init_mathlib()
 			mathlibs[libidx].available = 1;
 
 		// At this point the library load was successful, provide detail:
-		if (mathlibs[libidx].handle != NULL)
-		{
-			char lpath[PATH_MAX];
-			dlinfo(mathlibs[libidx].handle, RTLD_DI_ORIGIN, lpath);
-			printf("  loaded ok: %s/%s\n", lpath, mathlibs[libidx].lib);
-		}
-		else
-			printf("  loaded ok.\n");
+		pr_notice("%s: loaded\n", mathlibs[libidx].name);
 
 		// Set the default to the first one we find:
 		if (current_mathlib == NULL)
@@ -233,6 +236,23 @@ void init_mathlib()
 			// Otherwise close it for now and re-open on use.
 			close_mathlib(&mathlibs[libidx]);
 	}
+
+	if (!mathlibs[rc_config.mathlib_idx].available)
+		for (libidx = 0; libidx < num_mathlibs; libidx++)
+			if (mathlibs[libidx].available)
+			{
+				set_mathlib_interactive(NULL, &mathlibs[libidx]);
+				break;
+			}
+
+	if (!mathlibs[rc_config.mathlib_batch_idx].available)
+		for (libidx = 0; libidx < num_mathlibs; libidx++)
+			if (mathlibs[libidx].available)
+			{
+				rc_config.mathlib_batch_idx = libidx;
+				break;
+			}
+
 }
 
 
@@ -241,14 +261,25 @@ void init_mathlib()
 // Restore selection on open:
 void mathlib_config_init(rc_config_vars_t *v, char *line)
 {
-	if (rc_config.mathlib_idx < num_mathlibs &&
-		rc_config.mathlib_idx >= 0 &&
-		mathlibs[rc_config.mathlib_idx].available)
+	if (rc_config.mathlib_idx >= num_mathlibs || rc_config.mathlib_idx < 0)
 	{
-		set_mathlib_interactive(NULL, &mathlibs[rc_config.mathlib_idx]);
+		pr_warn("mathlib_idx out of range, clamping to zero: 0 !< %d !< %d\n",
+			rc_config.mathlib_idx, num_mathlibs);
+		rc_config.mathlib_idx = 0;
 	}
-	else
-		printf("Unable to set the preferred mathlib index to %d\n", rc_config.mathlib_idx);
+
+	if (rc_config.mathlib_batch_idx >= num_mathlibs || rc_config.mathlib_batch_idx < 0)
+	{
+		pr_warn("mathlib_batch_idx out of range, clamping to zero: 0 !< %d !< %d\n",
+			rc_config.mathlib_batch_idx, num_mathlibs);
+		rc_config.mathlib_batch_idx = 0;
+	}
+
+	if (mathlibs[rc_config.mathlib_idx].available)
+		set_mathlib_interactive(NULL, &mathlibs[rc_config.mathlib_idx]);
+	else if (!rc_config.first_run)
+		pr_err("%s was not detected: Unable to set the mathlib index to %d\n",
+		       mathlibs[rc_config.mathlib_idx].name, rc_config.mathlib_idx);
 }
 
 int mathlib_config_benchmark_parse(rc_config_vars_t *v, char *line)
@@ -284,7 +315,7 @@ void set_mathlib_interactive(GtkWidget *widget, mathlib_t *lib)
 
 	if (lib == NULL)
 	{
-		printf("set_mathlib_interactive: lib == NULL\n");
+		BUG("set_mathlib_interactive: lib == NULL\n");
 		return;
 	}
 
@@ -293,9 +324,8 @@ void set_mathlib_interactive(GtkWidget *widget, mathlib_t *lib)
 
 	if (!lib->available)
 	{
-		printf("set_mathlib_interactive[%d]: mathlib.available=0, skipping: %s\n",
-			getpid(),
-			lib->name);
+		pr_err("set_mathlib_interactive[%d]: mathlib.available=0, skipping: %s\n",
+		        getpid(), lib->name);
 
 		// If the library is unavailable for some reason then clear the one that was
 		// just set and reset the active one:
@@ -333,7 +363,7 @@ void set_mathlib_batch(GtkWidget *widget, mathlib_t *lib)
 
 	if (lib == NULL)
 	{
-		printf("set_mathlib_batch: lib == NULL\n");
+		BUG("set_mathlib_batch: lib == NULL\n");
 		return;
 	}
 
@@ -379,7 +409,7 @@ void set_mathlib_benchmark(GtkWidget *widget, mathlib_t *lib)
 						gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mathlibs[i].benchmark_widget), FALSE);
 			}
 
-		printf("%s: benchmark=%d\n", lib->name, lib->benchmark);
+		pr_info("%s: benchmark=%d\n", lib->name, lib->benchmark);
 	}
 }
 
@@ -469,7 +499,7 @@ void mathlib_help()
 		"== Math Libraries by Distribution ==\n"
 		"* CentOS: yum install atlas-devel openblas-devel\n"
 		"\n"
-		"* Debian/Ubuntu: apt-get install liblapacke libopenblas0-* libatlas3-base\n"
+		"* Debian/Ubuntu: apt-get install liblapacke libopenblas*-* libatlas3-base\n"
 		"    Note that you may need to use `alternatives` to select your\n"
 		"    openblas implementation. Some combinations of BLAS/LAPACK may\n"
 		"    or may not work together:\n"
@@ -620,7 +650,8 @@ void mathlib_benchmark(int slow)
 
 		while (calc_data.num_jobs >= 1)
 		{
-			printf("\nStarting %s benchmark (-j %d)\n", active_mathlib->name, calc_data.num_jobs);
+			pr_info("Starting %s benchmark (-j %d)\n",
+				active_mathlib->name, calc_data.num_jobs);
 
 			New_Frequency_Reset_Prev();
 			calc_data.fmhz_save = 0;
@@ -716,8 +747,8 @@ void mathlib_lock_intel(int locked_idx, int batch)
 	locked_lib = get_mathlib_by_idx(locked_idx);
 	if (locked_lib == NULL)
 	{
-		printf("mathlib_lock_intel: Cannot find library for mathlib idx=%d\n",
-			locked_idx);
+		BUG("mathlib_lock_intel: Cannot find library for mathlib idx=%d\n",
+		        locked_idx);
 		return;
 	}
 
@@ -727,18 +758,18 @@ void mathlib_lock_intel(int locked_idx, int batch)
 	if (!warned && !CHILD)
 	{
 		warned = 1;
-		printf( _("\n"
-		"* Notice: \"%s\" was selected, but you can only change to a\n"
-		"* different Intel MKL threading library by restarting xnec2c.\n"
-		"* \n"
-		"* However, you may continue to select any other Linear Algebra library\n"
-		"* without restart, including this same Intel MKL library.  All other\n"
-		"* Intel MKL threading libraries are disabled until restart because of an \n"
-		"* MKL limitation where the MKL threading library can only be dynamically\n"
-		"* linked only once per runtime.\n"
-		"* \n"
-		"* See Intel's documentation on mkl_set_threading_layer() for more detail.\n"),
-			 locked_lib->name);
+		pr_notice("\n"
+		        "* Notice: \"%s\" was selected, but you can only change to a\n"
+		        "* different Intel MKL threading library by restarting xnec2c.\n"
+		        "* \n"
+		        "* However, you may continue to select any other Linear Algebra library\n"
+		        "* without restart, including this same Intel MKL library.  All other\n"
+		        "* Intel MKL threading libraries are disabled until restart because of an \n"
+		        "* MKL limitation where the MKL threading library can only be dynamically\n"
+		        "* linked only once per runtime.\n"
+				"* \n"
+		        "* See Intel's documentation on mkl_set_threading_layer() for more detail.\n",
+		        locked_lib->name);
 	}
 
 	// Disable the other MKL threading options.
@@ -795,7 +826,7 @@ int32_t zgetrf(int32_t order, int32_t m, int32_t n, complex double *a, int32_t n
 
 	if (current_mathlib == NULL)
 	{
-		printf("zgetrf: current_mathlib is NULL, this should never happen.\n");
+		BUG("zgetrf: current_mathlib is NULL, this should never happen.\n");
 		return 1;
 	}
 
@@ -819,7 +850,8 @@ int32_t zgetrf(int32_t order, int32_t m, int32_t n, complex double *a, int32_t n
 		return factr_gauss_elim(n, a, (int32_t*)ip, ndim);
 	}
 	else
-		printf("%s: unsupported mathlib type %d\n", __func__, current_mathlib->type);
+		BUG("%s: unsupported mathlib type %d\n", __func__,
+			current_mathlib->type);
 
 	return 1;
 }
@@ -831,11 +863,11 @@ int32_t zgetrs(int32_t order, int32_t trans, int32_t lda, int32_t nrhs,
 
 	if (current_mathlib == NULL)
 	{
-		printf("zgetrs: current_mathlib is NULL, this should never happen.\n");
+		BUG("zgetrs: current_mathlib is NULL, this should never happen.\n");
 		return 1;
 	}
 
-	//printf("%s: type=%d typename=%s\n", mathfuncs[f_idx], current_mathlib->type, current_mathlib->name);
+	//pr_debug("%s: type=%d typename=%s\n", mathfuncs[f_idx], current_mathlib->type, current_mathlib->name);
 
 	if (current_mathlib->type == MATHLIB_ATLAS)
 	{
@@ -856,13 +888,15 @@ int32_t zgetrs(int32_t order, int32_t trans, int32_t lda, int32_t nrhs,
 	else if (current_mathlib->type == MATHLIB_NEC2)
 	{
 		if (lda != ldb)
-			printf("zgetrs warning: lda(%d) != ldb(%d)\n", lda, ldb);
+			BUG("zgetrs warning: lda(%d) != ldb(%d)\n", lda,
+				ldb);
 
 		// use the original NEC2 function
 		return solve_gauss_elim(lda, a, (int32_t*)ip, b, ndim);
 	}
 	else
-		printf("%s: unsupported mathlib type %d\n", __func__, current_mathlib->type);
+		BUG("%s: unsupported mathlib type %d\n", __func__,
+			current_mathlib->type);
 
 	return 1;
 }
@@ -889,7 +923,7 @@ void mathlib_mkl_set_threading(mathlib_t *lib, int code)
 	*(void **) (&mkl_set_threading_layer) = dlsym(lib->handle, "MKL_Set_Threading_Layer");
 	if (mkl_set_threading_layer == NULL)
 	{
-		printf("dlsym(mkl_set_threading_layer): %s\n", dlerror());
+		pr_err("dlsym(mkl_set_threading_layer): %s\n", dlerror());
 		return;
 	}
 
