@@ -73,6 +73,9 @@ void fr_plots_init()
 
   for (idx = 0; idx < calc_data.ngraph * calc_data.FR_cards; idx++)
   {
+	  if (FR_PLOT_T_IS_VALID(&fr_plots[idx]))
+		  continue;
+
 	  // Set the plot position
 	  fr_plots[idx].posn = idx / calc_data.FR_cards;
 
@@ -126,9 +129,10 @@ static inline void fr_plot_sync_widths(fr_plot_t *fr_plot)
 			current = get_plot_rect(fr_plot->posn, fr);
 			if (current == NULL)
 			{
-				printf("fr_plot_sync_widths, current == NULL: "
-						"calc_data.FR_cards=%d calc_data.ngraph=%d fr=%d posn=%d valid=%d\n",
-					calc_data.FR_cards, calc_data.ngraph, fr, fr_plot->posn, FR_PLOT_T_IS_VALID(fr_plot));
+				BUG("fr_plot_sync_widths, current == NULL: calc_data.FR_cards=%d calc_data.ngraph=%d fr=%d posn=%d valid=%d\n",
+					calc_data.FR_cards, calc_data.ngraph,
+					fr, fr_plot->posn,
+					FR_PLOT_T_IS_VALID(fr_plot));
 				return;
 			}
 			r = get_plot_rect(posn, fr);
@@ -141,17 +145,11 @@ static inline void fr_plot_sync_widths(fr_plot_t *fr_plot)
 
 void print_fr_plot(fr_plot_t *p)
 {
-	printf("fr_plot[posn=%d fr=%d] rect[x=%d y=%d, w=%d h=%d] freq[min=%4.2f max=%4.2f n=%d]\n",
-		p->posn,
-		p->fr,
-		p->plot_rect.x,
-		p->plot_rect.y,
-		p->plot_rect.width,
-		p->plot_rect.height,
-		p->freq_loop_data->min_freq,
-		p->freq_loop_data->max_freq,
-		p->freq_loop_data->freq_steps
-		);
+	pr_debug("fr_plot[posn=%d fr=%d] rect[x=%d y=%d, w=%d h=%d] freq[min=%4.2f max=%4.2f n=%d]\n",
+		p->posn, p->fr, p->plot_rect.x, p->plot_rect.y,
+		p->plot_rect.width, p->plot_rect.height,
+		p->freq_loop_data->min_freq, p->freq_loop_data->max_freq,
+		p->freq_loop_data->freq_steps);
 }
 
 /* draw_text:
@@ -248,11 +246,12 @@ void draw_text(cairo_t *cr, GtkWidget *widget,
   static void
 Display_Frequency_Data( void )
 {
-  int pol, fstep;
-  double vswr, gamma;
-  double zrpro2, zrmro2, zimag2;
+  int fstep;
+  double vswr;
   char txt[12];
 
+  measurement_t meas;
+
   if( isFlagClear(PLOT_ENABLED) ) return;
 
   /* Limit freq stepping to freq_steps FIXME */
@@ -264,39 +263,19 @@ Display_Frequency_Data( void )
   if( fstep >= calc_data.steps_total )
     fstep = calc_data.steps_total;
 
-  /* Polarization type */
-  pol = calc_data.pol_type;
+  meas_calc(&meas, fstep);
 
-  /* Index to gtot buffer where max gain
-   * occurs for given polarization type */
-  if( isFlagSet(ENABLE_RDPAT) )
-  {
-    /* Max gain for given polarization type */
-    int mgidx = rad_pattern[fstep].max_gain_idx[pol];
-    double gmax = rad_pattern[fstep].gtot[mgidx] +
-      Polarization_Factor(pol, fstep, mgidx);
-
-    /* Display max gain */
-    snprintf( txt, 7, "%5.1f", gmax );
-    gtk_entry_set_text( GTK_ENTRY(Builder_Get_Object(
-            freqplots_window_builder, "freqplots_maxgain_entry")), txt );
-
-  } /* isFlagSet(ENABLE_RDPAT) */
+  /* Display max gain */
+  snprintf( txt, 7, "%5.1f", meas.gain_max );
+  gtk_entry_set_text( GTK_ENTRY(Builder_Get_Object(
+          freqplots_window_builder, "freqplots_maxgain_entry")), txt );
 
   /* Display frequency */
   snprintf( txt, 11, "%9.3f", (double)calc_data.freq_mhz );
   gtk_entry_set_text( GTK_ENTRY(Builder_Get_Object(
           freqplots_window_builder, "freqplots_fmhz_entry")), txt );
 
-  /* Calculate VSWR */
-  zrpro2 = (double)creal( netcx.zped ) + calc_data.zo;
-  zrpro2 *= zrpro2;
-  zrmro2 = (double)creal( netcx.zped ) - calc_data.zo;
-  zrmro2 *= zrmro2;
-  zimag2 = (double)cimag( netcx.zped );
-  zimag2 *= zimag2;
-  gamma = sqrt( (zrmro2 + zimag2)/(zrpro2 + zimag2) );
-  vswr = (1+gamma)/(1-gamma);
+  vswr = meas.vswr;
   if( vswr > 999.0 )
     vswr = 999.0;
 
@@ -306,12 +285,12 @@ Display_Frequency_Data( void )
           freqplots_window_builder, "freqplots_vswr_entry")), txt );
 
   /* Display Z real */
-  snprintf( txt, 7, "%5.1f", (double)creal( netcx.zped ) );
+  snprintf( txt, 7, "%5.1f", (double)meas.zreal);
   gtk_entry_set_text( GTK_ENTRY(Builder_Get_Object(
           freqplots_window_builder, "freqplots_zreal_entry")), txt );
 
   /* Display Z imaginary */
-  snprintf( txt, 7, "%5.1f", (double)cimag( netcx.zped ) );
+  snprintf( txt, 7, "%5.1f", (double)meas.zimag);
   gtk_entry_set_text( GTK_ENTRY(Builder_Get_Object(
           freqplots_window_builder, "freqplots_zimag_entry")), txt );
 
@@ -596,7 +575,7 @@ Plot_Horizontal_Scale(
 {
   int idx, order;
   double hstep = 1.0;
-  char value[16], format[8];
+  char value[16], format[16];
 
   /* Abort if not enough values to plot */
   if( nval <= 1 ) return;
@@ -608,9 +587,14 @@ Plot_Horizontal_Scale(
   /* Use order of horizontal step to determine format of print */
   double ord = log10( fabs(hstep + 0.0000001) );
   order = (int)ord;
+
+  if (!rc_config.freqplots_round_x_axis)
+	  order--;
+
   if( order > 0 )  order = 0;
   if( order < -9 ) order = -9;
-  snprintf( format, 7, "%%.%df", 1-order );
+
+  snprintf( format, 16, "%%.%df", 1-order );
 
   /* Draw horizontal scale values */
   for( idx = 0; idx < nval; idx++ )
@@ -651,7 +635,7 @@ Plot_Vertical_Scale(
   int idx, yps;
   int min_order, max_order, order;
   double vstep = 1.0;
-  char value[16], format[6];
+  char value[16], format[16];
 
   /* Abort if not enough values to plot */
   if( nval <= 1 ) return;
@@ -679,7 +663,7 @@ Plot_Vertical_Scale(
   order = ( max_order > min_order ? max_order : min_order );
   if( order > 3 ) order = 3;
   if( order < 0 ) order = 0;
-  snprintf( format, 6, "%%.%df", (3-order) );
+  snprintf( format, 16, "%%.%df", (3-order) );
 
   /* Draw vertical scale values */
   for( idx = 0; idx < nval; idx++ )
@@ -771,6 +755,8 @@ void draw_poly(
 /* Draw_Graph()
  *
  * Plots a graph of a vs b
+ * nval: number of points to plot
+ * nval_max: number of points in the whole graph in case nval<nval_max
  */
   static void
 Draw_Graph(
@@ -780,9 +766,9 @@ Draw_Graph(
     double *a, double *b,
     double amax, double amin,
     double bmax, double bmin,
-    int nval, int side )
+    int nval, int nval_max, int side )
 {
-  double ra;
+  double ra, rb;
   int idx;
   GdkPoint *points = NULL;
   char s[20];
@@ -792,14 +778,14 @@ Draw_Graph(
 
   /* Range of values to plot */
   ra = amax - amin;
+  rb = bmax - bmin;
 
   /* Calculate points to plot */
-  mem_alloc( (void **) &points, (size_t)calc_data.steps_total * sizeof(GdkPoint),
+  mem_alloc( (void **) &points, (size_t)nval * sizeof(GdkPoint),
       "in Draw_Graph()" );
   if( points == NULL )
   {
-    fprintf( stderr, _("xnec2c: Draw_Graph():"
-        "memory allocation for points failed\n") );
+    pr_err("memory allocation for points failed\n");
     Stop( _("Draw_Graph():"
           "Memory allocation for points failed"), ERR_OK );
     return;
@@ -808,7 +794,7 @@ Draw_Graph(
   int min_idx = 0, max_idx = 0;
   for( idx = 0; idx < nval; idx++ )
   {
-    points[idx].x = rect->x + (int)((double)rect->width * idx/(nval-1) + 0.5);
+    points[idx].x = rect->x + (int)((double)rect->width * (b[idx]-bmin)/rb + 0.5);
     points[idx].y = rect->y + (int)( (double)rect->height *
         (amax-a[idx]) / ra + 0.5 );
 
@@ -917,7 +903,7 @@ Plot_Graph(
 	// pango_text_size() above:
 
 	// Space between vertical scale lines across the X axis:
-	px_per_vert_scale = pad_x_scale_text*1.5;
+	px_per_vert_scale = 75;
 
 	// Space between horizontal scale lines down the Y axis:
 	px_per_horiz_scale = pad_y_bottom_scale_text*3;
@@ -1074,12 +1060,10 @@ Plot_Graph(
 
 
 		// Offset is the offset into the value arrays: 
-		// Break if there are no more values to plot because
+		// Skip plot below if there are no more values to plot because
 		// it has plotted all of them or the values are still
 		// being calculated:
-		int maxidx = calc_data.freq_step - offset;
-		if (maxidx <= 0)
-			break;
+		int maxidx = nx - offset;
 
 		// Clamp the number of index to be plotted if there
 		// are some available to plot in the next FR card which
@@ -1088,8 +1072,21 @@ Plot_Graph(
 			maxidx = fr_plot->freq_loop_data->freq_steps;
 
 		/* Draw plotting frame */
-		double min_fscale = fr_plot->freq_loop_data->min_freq;
-		double max_fscale = fr_plot->freq_loop_data->max_freq;
+
+		// Scale the X-axis frequency values to nice round numbers:
+		fr_plot->min_fscale = fr_plot->freq_loop_data->min_freq;
+		fr_plot->max_fscale = fr_plot->freq_loop_data->max_freq;
+
+		if (rc_config.freqplots_round_x_axis)
+			Fit_to_Scale( &fr_plot->max_fscale, &fr_plot->min_fscale, &n_vert_scale );
+
+		// Always label at least 2 frequencies, the first and last:
+		if (n_vert_scale < 2)
+			n_vert_scale = 2;
+
+		// local shorthand variables
+		double min_fscale = fr_plot->min_fscale;
+		double max_fscale = fr_plot->max_fscale;
 
 		Draw_Plotting_Frame( cr, titles,
 			plot_rect, n_horiz_scale, n_vert_scale);
@@ -1102,7 +1099,7 @@ Plot_Graph(
 			plot_rect->width,
 			max_fscale, min_fscale, n_vert_scale);
 
-		if (y_left != NULL)
+		if (maxidx > 0 && y_left != NULL)
 		{
 			Draw_Graph(
 				cr,
@@ -1111,11 +1108,11 @@ Plot_Graph(
 				y_left+offset, x+offset,
 				max_y_left, min_y_left,
 				max_fscale, min_fscale,
-				maxidx,
+				maxidx, fr_plot->freq_loop_data->freq_steps,
 				LEFT );
 		}
 
-		if (y_right != NULL)
+		if (maxidx > 0 && y_right != NULL)
 		{
 			Draw_Graph(
 				cr,
@@ -1124,7 +1121,7 @@ Plot_Graph(
 				y_right+offset, x+offset,
 				max_y_right, min_y_right,
 				max_fscale, min_fscale,
-				maxidx,
+				maxidx, fr_plot->freq_loop_data->freq_steps,
 				RIGHT);
 		}
 
@@ -1133,8 +1130,8 @@ Plot_Graph(
 		* The +/- 0.001 is to adjust for floating-point error, 
 		* for example: freq_mhz=148.000000 !<= max_fscale=147.999996 */
 		if( isFlagSet(FREQ_LOOP_DONE) && isFlagSet(PLOT_FREQ_LINE)
-			&& calc_data.freq_mhz >= min_fscale - 0.001 
-			&& calc_data.freq_mhz <= max_fscale + 0.001)
+			&& calc_data.freq_mhz >= min_fscale - 1e-6
+			&& calc_data.freq_mhz <= max_fscale + 1e-6)
 		{
 			double freq_x;
 
@@ -1236,13 +1233,12 @@ Plot_Graph_Smith(
   cairo_set_source_rgb( cr, MAGENTA );
 
   /* Calculate points to plot */
-  mem_alloc( (void **)&points, (size_t)calc_data.steps_total * sizeof(GdkPoint),
+  mem_alloc( (void **)&points, (size_t)nc * sizeof(GdkPoint),
       "in Draw_Graph()" );
 
   if( points == NULL )
   {
-    fprintf( stderr, _("xnec2c: Draw_Graph():"
-        "memory allocation for points failed\n") );
+    pr_err("memory allocation for points failed\n");
     Stop( _("Draw_Graph():"
           "Memory allocation for points failed"), ERR_OK );
     return;
@@ -1251,6 +1247,10 @@ Plot_Graph_Smith(
   for( idx = 0; idx < nc; idx++ )
   {
     Calculate_Smith( fa[idx], fb[idx], calc_data.zo, &re, &im );
+
+    // flip plot vertically because negative imaginary is the bottom half    
+    im = -im;
+
     points[idx].x = x0 + (gint)( re * scale / 2 );
     points[idx].y = y0 + (gint)( im * scale / 2 );
         cairo_rectangle( cr,
@@ -1269,6 +1269,10 @@ Plot_Graph_Smith(
   {
     cairo_set_source_rgb( cr, GREEN );
     Calculate_Smith( creal(netcx.zped), cimag(netcx.zped), calc_data.zo, &re, &im );
+
+    // flip plot vertically because negative imaginary is the bottom half    
+    im = -im;
+
     x = x0 + (gint)( re * scale / 2 );
     y = y0 + (gint)( im * scale / 2 );
     cairo_rectangle( cr, x - 4, y - 4, 8.0, 8.0 );
@@ -1299,7 +1303,7 @@ _Plot_Frequency_Data( cairo_t *cr )
 
   int idx,
     posn,  /* Position num of plot in drawingarea */
-    fstep; /* Freq step number */
+    num_fsteps; /* Freq step number */
 
   static double
     *gmax     = NULL, /* Max gain buffer */
@@ -1309,8 +1313,7 @@ _Plot_Frequency_Data( cairo_t *cr )
     *gdir_phi = NULL, /* Direction in phi of gain */
     *fbratio  = NULL; /* Direction in phi of gain */
 
-  /* Used to calculate net gain */
-  double Zr, Zo, Zi;
+  measurement_t meas;
 
   fr_plots_init();
 
@@ -1324,9 +1327,12 @@ _Plot_Frequency_Data( cairo_t *cr )
       (double)freqplots_width,
       (double)freqplots_height );
   cairo_fill( cr );
+  
+  /* Limit freq stepping to last freq step */
+  num_fsteps = calc_data.last_step + 1;
 
   /* Abort if plotting is not possible FIXME */
-  if( (calc_data.freq_step < 1) || isFlagClear(FREQ_LOOP_READY) ||
+  if( (num_fsteps < 0) || isFlagClear(FREQ_LOOP_READY) ||
       (isFlagClear(FREQ_LOOP_RUNNING) && isFlagClear(FREQ_LOOP_DONE)) ||
 
       (isFlagClear(PLOT_GMAX)         &&
@@ -1347,17 +1353,13 @@ _Plot_Frequency_Data( cairo_t *cr )
   /* Graph position */
   posn = 0;
 
-  /* Limit freq stepping to last freq step */
-  fstep = calc_data.last_step + 1;
-
   /* Plot max gain vs frequency, if possible */
   if( isFlagSet(PLOT_GMAX) && isFlagSet(ENABLE_RDPAT) )
   {
-    int nth, nph, pol;
     gboolean no_fbr;
 
     /* Allocate max gmax and directions */
-    size_t mreq = (size_t)fstep * sizeof(double);
+    size_t mreq = (size_t) num_fsteps * sizeof(double);
     mem_realloc( (void **)&gmax,     mreq, "in plot_freqdata.c" );
     mem_realloc( (void **)&gdir_tht, mreq, "in plot_freqdata.c" );
     mem_realloc( (void **)&gdir_phi, mreq, "in plot_freqdata.c" );
@@ -1369,79 +1371,23 @@ _Plot_Frequency_Data( cairo_t *cr )
     no_fbr = FALSE;
 
     /* Polarization type and impedance */
-    pol = calc_data.pol_type;
-    Zo = calc_data.zo;
 
     /* When freq loop is done, calcs are done for all freq steps */
-    for( idx = 0; idx < fstep; idx++ )
+    for( idx = 0; idx < num_fsteps; idx++ )
     {
-      double fbdir;
-      int fbidx, mgidx;
+	  meas_calc(&meas, idx);
 
-      /* Index to gtot buffer where max gain
-       * occurs for given polarization type */
-      mgidx = rad_pattern[idx].max_gain_idx[pol];
+	  gmax[idx] = meas.gain_max;
+	  if( isFlagSet(PLOT_NETGAIN) )
+		  netgain[idx] = meas.gain_net;
+	  gdir_tht[idx] = meas.gain_max_theta;
+	  gdir_phi[idx] = meas.gain_max_phi;
 
-      /* Max gain for given polarization type */
-      gmax[idx] = rad_pattern[idx].gtot[mgidx] + Polarization_Factor(pol, idx, mgidx);
-
-      /* Net gain if selected */
-      if( isFlagSet(PLOT_NETGAIN) )
-      {
-        Zr = impedance_data.zreal[idx];
-        Zi = impedance_data.zimag[idx];
-        netgain[idx] = gmax[idx] +
-          10.0 * log10( 4.0 * Zr * Zo / (pow(Zr + Zo, 2.0) + pow( Zi, 2.0 )) );
-      }
-
-      /* Radiation angle/phi where max gain occurs */
-      gdir_tht[idx] = 90.0 - rad_pattern[idx].max_gain_tht[pol];
-      gdir_phi[idx] = rad_pattern[idx].max_gain_phi[pol];
-
-      /* Find F/B ratio if possible or net gain not required */
-      if( no_fbr || isFlagSet(PLOT_NETGAIN) )
-        continue;
-
-      /* Find F/B direction in theta */
-      fbdir = 180.0 - rad_pattern[idx].max_gain_tht[pol];
-      if( fpat.dth == 0.0 ) nth = 0;
-      else nth = (int)( fbdir/fpat.dth + 0.5 );
-
-      /* If the antenna is modelled over ground, then use the same
-       * theta as the max gain direction, relying on phi alone to
-       * take us to the back. Patch supplied by Rik van Riel AB1KW
-       */
-      if( (nth >= fpat.nth) || (nth < 0) )
-      {
-        fbdir = rad_pattern[idx].max_gain_tht[pol];
-        if( fpat.dth == 0.0 )
-          nth = 0;
-        else
-          nth = (int)( fbdir / fpat.dth + 0.5 );
-      }
-
-      /* Find F/B direction in phi */
-      fbdir = gdir_phi[idx] + 180.0;
-      if( fbdir >= 360.0 ) fbdir -= 360.0;
-      nph = (int)( fbdir/fpat.dph + 0.5 );
-
-      /* No F/B calc. possible if no phi step at +180 from max gain */
-      if( (nph >= fpat.nph) || (nph < 0) )
-      {
-        no_fbr = TRUE;
-        continue;
-      }
-
-      /* Index to gtot buffer for gain in back direction */
-      fbidx = nth + nph*fpat.nth;
-
-      /* Front to back ratio */
-      fbratio[idx]  = pow( 10.0, gmax[idx] / 10.0 );
-      fbratio[idx] /= pow( 10.0,
-          (rad_pattern[idx].gtot[fbidx] + Polarization_Factor(pol, idx, fbidx)) / 10.0 );
-      fbratio[idx] = 10.0 * log10( fbratio[idx] );
-      rad_pattern[idx].fbratio = fbratio[idx];
-    } /* for( idx = 0; idx < fstep; idx++ ) */
+	  if (meas.fb_ratio >= 0)
+		  fbratio[idx] = meas.fb_ratio;
+	  else
+		  no_fbr = TRUE;
+    }
 
     /*** Plot gain and f/b ratio (if possible) graph(s) */
     if( no_fbr || isFlagSet(PLOT_NETGAIN) )
@@ -1452,20 +1398,15 @@ _Plot_Frequency_Data( cairo_t *cr )
       {
         titles[1] = _("Max Gain & Net Gain vs Frequency");
         titles[2] = _("Net Gain dBi");
-        if( fstep > 1 )
-          Plot_Graph( cr,
-			  gmax, netgain,
-			  save.freq,
-			  fstep,
-              titles, posn++ );
+        if (num_fsteps > 0)
+          Plot_Graph(cr, gmax, netgain, save.freq, num_fsteps, titles, posn++);
       }
       else
       {
         titles[1] = _("Max Gain & F/B Ratio vs Frequency");
         titles[2] = "        ";
-        if( fstep > 1 )
-          Plot_Graph( cr, gmax, NULL, save.freq, fstep,
-              titles, posn++ );
+        if (num_fsteps > 0)
+          Plot_Graph(cr, gmax, NULL, save.freq, num_fsteps, titles, posn++);
       }
     }
     else
@@ -1474,9 +1415,8 @@ _Plot_Frequency_Data( cairo_t *cr )
       titles[0] = _("Raw Gain dBi");
       titles[1] = _("Max Gain & F/B Ratio vs Frequency");
       titles[2] = _("F/B Ratio dB");
-      if( fstep > 1 )
-        Plot_Graph( cr, gmax, fbratio, save.freq, fstep,
-            titles, posn++ );
+      if (num_fsteps > 0)
+        Plot_Graph(cr, gmax, fbratio, save.freq, num_fsteps, titles, posn++);
     }
 
     /* Plot max gain direction if enabled */
@@ -1486,9 +1426,9 @@ _Plot_Frequency_Data( cairo_t *cr )
       titles[0] = _("Rad Angle - deg");
       titles[1] = _("Max Gain Direction vs Frequency");
       titles[2] = _("Phi - deg");
-      if( fstep > 1 )
-        Plot_Graph( cr, gdir_tht, gdir_phi, save.freq, fstep,
-            titles, posn++ );
+      if (num_fsteps > 0)
+        Plot_Graph(cr, gdir_tht, gdir_phi, save.freq, num_fsteps, titles,
+                   posn++);
     }
 
   } /* if( isFlagSet(PLOT_GMAX) && isFlagSet(ENABLE_RDPAT) ) */
@@ -1501,48 +1441,42 @@ _Plot_Frequency_Data( cairo_t *cr )
     titles[1] = _("Gain in Viewer Direction vs Frequency");
 
     /* Allocate viewer gain buffer */
-    size_t mreq = (size_t)fstep * sizeof(double);
+    size_t mreq = (size_t) num_fsteps * sizeof(double);
     mem_realloc( (void **)&vgain, mreq, "in plot_freqdata.c" );
 
     /* Calcs are done for all freq steps */
-    for( idx = 0; idx < fstep; idx++ )
+    for( idx = 0; idx < num_fsteps; idx++ )
       vgain[idx] = Viewer_Gain( structure_proj_params, idx );
 
     /* Plot net gain if selected */
     if( isFlagSet(PLOT_NETGAIN) )
     {
-      mreq = (size_t)fstep * sizeof(double);
+      mreq = (size_t) num_fsteps * sizeof(double);
       mem_realloc( (void **)&netgain, mreq, "in plot_freqdata.c" );
 
-      Zo = calc_data.zo;
-      for( idx = 0; idx < fstep; idx++ )
+      for( idx = 0; idx < num_fsteps; idx++ )
       {
-        Zr = impedance_data.zreal[idx];
-        Zi = impedance_data.zimag[idx];
-        netgain[idx] = vgain[idx] +
-          10.0 * log10( 4.0 * Zr * Zo / (pow(Zr + Zo, 2.0 ) + pow(Zi, 2.0)) );
+		  meas_calc(&meas, idx);
+		  netgain[idx] = meas.gain_viewer_net;
       }
 
       /* Plot net gain if selected */
       titles[2] = _("Net gain dBi");
-      if( fstep > 1 )
-        Plot_Graph( cr, vgain, netgain, save.freq, fstep,
-            titles, posn++ );
+      if (num_fsteps > 0)
+        Plot_Graph(cr, vgain, netgain, save.freq, num_fsteps, titles, posn++);
     } /* if( isFlagSet(PLOT_NETGAIN) ) */
     else
     {
       titles[2] = "        ";
-      if( fstep > 1 )
-        Plot_Graph( cr, vgain, NULL, save.freq, fstep,
-            titles, posn++ );
+      if (num_fsteps > 0)
+        Plot_Graph(cr, vgain, NULL, save.freq, num_fsteps, titles, posn++);
     }
   } /* isFlagSet(PLOT_GVIEWER) && isFlagSet(ENABLE_RDPAT) */
 
   /* Plot VSWR vs freq */
   if( isFlagSet(PLOT_VSWR) )
   {
-    double *vswr = NULL, *s11 = NULL, gamma;
-    double zrpro2, zrmro2, zimag2;
+    double *vswr = NULL, *s11 = NULL;
 
     /* Plotting frame titles */
     titles[0] = _("VSWR");
@@ -1559,42 +1493,36 @@ _Plot_Frequency_Data( cairo_t *cr )
 
 
     /* Calculate VSWR */
-    mem_alloc( (void **) &vswr, (size_t)calc_data.steps_total * sizeof(double),
+    mem_alloc( (void **) &vswr, (size_t)(calc_data.steps_total+1) * sizeof(double),
         "in Plot_Frequency_Data()" );
 
-    mem_alloc( (void **) &s11, (size_t)calc_data.steps_total * sizeof(double),
+    mem_alloc( (void **) &s11, (size_t)(calc_data.steps_total+1) * sizeof(double),
         "in Plot_Frequency_Data()" );
 
     if( vswr == NULL || s11 == NULL)
     {
-      fprintf( stderr, _("xnec2c: Plot_Frequency_Data():"
-          "memory allocation for vswr/s11 failed\n") );
+      pr_err("memory allocation for vswr/s11 failed\n");
       Stop( _("Plot_Frequency_Data():"
             "Memory allocation for vswr/s11 failed"), ERR_OK );
       return;
     }
 
-    for(idx = 0; idx < fstep; idx++ )
+    for(idx = 0; idx < num_fsteps; idx++ )
     {
-      zrpro2 = impedance_data.zreal[idx] + calc_data.zo;
-      zrpro2 *= zrpro2;
-      zrmro2 = impedance_data.zreal[idx] - calc_data.zo;
-      zrmro2 *= zrmro2;
-      zimag2 = impedance_data.zimag[idx] * impedance_data.zimag[idx];
-      gamma = sqrt( (zrmro2 + zimag2) / (zrpro2 + zimag2) );
+      meas_calc(&meas, idx);
       
-      vswr[idx] = (1.0 + gamma) / (1.0 - gamma);
+      vswr[idx] = meas.vswr;
       
       if (rc_config.freqplots_s11)
-          s11[idx] = 20*log10( gamma );
+          s11[idx] = meas.s11;
 
       if (rc_config.freqplots_clamp_vswr && vswr[idx] > 10.0 )
           vswr[idx] = 10.0;
     }
 
-    if( fstep > 1 )
-      Plot_Graph( cr, vswr, (rc_config.freqplots_s11 ? s11 : NULL), save.freq, fstep,
-          titles, posn++ );
+    if (num_fsteps > 0)
+      Plot_Graph(cr, vswr, (rc_config.freqplots_s11 ? s11 : NULL), save.freq,
+                 num_fsteps, titles, posn++);
 
     free_ptr( (void **)&vswr );
     free_ptr( (void **)&s11);
@@ -1607,10 +1535,9 @@ _Plot_Frequency_Data( cairo_t *cr )
     titles[0] = _("Z-real");
     titles[1] = _("Impedance vs Frequency");
     titles[2] = _("Z-imag");
-    if( fstep > 1 )
-      Plot_Graph( cr,
-          impedance_data.zreal, impedance_data.zimag, save.freq,
-          fstep, titles, posn++ );
+    if (num_fsteps > 0)
+      Plot_Graph(cr, impedance_data.zreal, impedance_data.zimag, save.freq,
+                 num_fsteps, titles, posn++);
 
   } /* if( isFlagSet(PLOT_ZREAL_ZIMAG) ) */
 
@@ -1621,19 +1548,18 @@ _Plot_Frequency_Data( cairo_t *cr )
     titles[0] = _("Z-magn");
     titles[1] = _("Impedance vs Frequency");
     titles[2] = _("Z-phase");
-    if( fstep > 1 )
-      Plot_Graph( cr, impedance_data.zmagn, impedance_data.zphase,
-          save.freq, fstep, titles, posn++ );
+    if (num_fsteps > 0)
+      Plot_Graph(cr, impedance_data.zmagn, impedance_data.zphase, save.freq,
+                 num_fsteps, titles, posn++);
 
   } /* if( isFlagSet(PLOT_ZREAL_ZIMAG) ) */
 
   /* Plot smith chart */
   if( isFlagSet(PLOT_SMITH) )
   {
-    if( fstep > 1 )
-      Plot_Graph_Smith( cr,
-          impedance_data.zreal, impedance_data.zimag, save.freq,
-		  fstep, posn++ );
+    if (num_fsteps > 0)
+      Plot_Graph_Smith(cr, impedance_data.zreal, impedance_data.zimag,
+                       save.freq, num_fsteps, posn++);
 
   } /* if( isFlagSet(PLOT_SMITH) ) */
 
@@ -1752,8 +1678,9 @@ _Set_Frequency_On_Click( GdkEvent *e)
 	  return;
   }
 
-  double min_fscale = fr_plot->freq_loop_data->min_freq;
-  double max_fscale = fr_plot->freq_loop_data->max_freq;
+  // local shorthand variables
+  double min_fscale = fr_plot->min_fscale;
+  double max_fscale = fr_plot->max_fscale;
 
   /* Width of plot bounding rectangle */
   w = fr_plot->plot_rect.width;
@@ -1784,10 +1711,10 @@ _Set_Frequency_On_Click( GdkEvent *e)
 	  else if (motion_event->state & GDK_BUTTON3_MASK) button = 3;
   }
 
-  /*
   // event types: https://gitlab.gnome.org/GNOME/gtk/-/blob/gtk-3-24/gdk/gdkevents.h#L309
   // Enable this for mouse debugging:
-  printf("mouse click[type=%d pos=%4.1f,%4.1f button=%d(%d)]: ",
+  /*
+  pr_debug("mouse click[type=%d pos=%4.1f,%4.1f button=%d(%d)]\n",
 	  e->type,
 	  button_event->x, button_event->y,
 	  button, button_event->button);
@@ -1870,32 +1797,30 @@ _Set_Frequency_On_Click( GdkEvent *e)
 		break;
 
 		default: 
-			printf("mouse button: unknown button %d\n", button);
+			pr_debug("mouse button: unknown button %d\n", button);
 			return;
 
   } /* switch( button_event->button ) */
 
   if (set_fmhz)
   {
-  /* Round frequency to nearest 1 kHz */
-  int ifmhz = ( fmhz * 1000.0 + 0.5 );
-  fmhz = ifmhz / 1000.0;
+	  /* Round frequency to nearest 1 kHz */
+	  int ifmhz = ( fmhz * 1e6 + 0.5 );
+	  fmhz = ifmhz / 1e6;
 
-  /* Save frequency for later use when the graph plots after the NEC2 run */
-  calc_data.fmhz_save = fmhz;
+	  /* Save frequency for later use when the graph plots after the NEC2 run */
+	  calc_data.fmhz_save = fmhz;
 
-  /* Set frequency spinbuttons on new freq */
-  if( fmhz != gtk_spin_button_get_value(mainwin_frequency) )
-  {
-    gtk_spin_button_set_value( mainwin_frequency, fmhz );
-    if( isFlagSet(DRAW_ENABLED) )
-      gtk_spin_button_set_value( rdpattern_frequency, fmhz );
-  }
-  else /* Replot data */
-  {
-    calc_data.freq_mhz = fmhz;
-    g_idle_add( Redo_Currents, NULL );
-  }
+	  /* Set frequency spinbuttons on new freq */
+	  if( fmhz != calc_data.freq_mhz )
+	  {
+		gtk_spin_button_set_value( mainwin_frequency, fmhz );
+		if( isFlagSet(DRAW_ENABLED) )
+		  gtk_spin_button_set_value( rdpattern_frequency, fmhz );
+
+		calc_data.freq_mhz = fmhz;
+		g_idle_add( Redo_Currents, NULL );
+	  }
   }
 
   // Only redraw the plots window if we hold the lock
