@@ -17,11 +17,19 @@
  *    https://www.xnec2c.org/
  */
 
+#include "config.h"
+
+#ifdef HAVE_BACKTRACE
 #include <execinfo.h>
+#endif
 
 #include "utils.h"
 #include "shared.h"
 
+char **_get_backtrace();
+void _print_backtrace(char **strings);
+
+
 /*------------------------------------------------------------------------*/
 
 /*  usage()
@@ -32,12 +40,27 @@
   void
 usage(void)
 {
-  fprintf( stderr,
-      _("Usage: xnec2c <input-file-name>\n"
-        "              [-i <input-file-name>]\n"
-        "              [-j <number of processors in SMP machine>]\n"
-        "              [-h: print this usage information and exit]\n"
-        "              [-v: print xnec2c version number and exit]\n") );
+  fprintf(stderr, "Usage: xnec2c [options] [<input-file-name>]\n"
+		"  -i|--input <input-file-name>\n"
+		"  -j|--jobs  <number of processors in SMP machine> (-j0 disables forking)\n"
+		"  -b|--batch:        enable batch mode, exit after the frequency loop runs\n"
+		"     --optimize:     Activate the optimizer immediately.\n"
+		"  -P|--no-pthreads:  disable pthreads and use the GTK loop for debugging\n"
+		"  -h|--help:         print usage information and exit\n"
+		"  -V|--version:      print xnec2c version number and exit\n"
+		"  -v|--verbose:      increase verbosity, can be specified multiple times\n"
+		"  -d|--debug:        enable debug output (-dd includes backtraces)\n"
+		"  -d|--quiet:        suppress debug/verbose output\n"
+		"\n"
+		"The following arguments write to an output file after the frequency loop\n"
+		"completes.  These are useful to combine with --batch; If you wish to specify\n"
+		"filenames to write without --batch mode then enable the File->Optimizer\n"
+		"Settings or the files you specify on the command line will not be written.\n"
+		"\n"
+		"  --write-csv <filename>\n"
+		"  --write-s1p <filename>\n"
+		"  --write-s2p-max-gain <filename>\n"
+		"  --write-s2p-viewer-gain <filename>\n");
 
 } /* end of usage() */
 
@@ -51,7 +74,7 @@ int Notice(char *title, char *message,  GtkButtonsType buttons)
 		buttons,
 		"%s", title);
 
-	printf("\n=== Notice: %s ===\n%s\n\n", title, message);
+	pr_notice("\n=== Notice: %s ===\n%s\n\n", title, message);
 	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(notice), "%s", message);
 
 	response = gtk_dialog_run(GTK_DIALOG(notice));
@@ -69,14 +92,14 @@ Stop( char *mesg, int err )
 {
   GtkBuilder *builder;
 
+  pr_err("Stop: %s\n", mesg);
   /* For child processes */
   if( CHILD )
   {
-    fprintf( stderr, "%s\n", mesg );
+    pr_err("%s\n", mesg);
     if( err )
     {
-      fprintf( stderr,
-          _("xnec2c: fatal: child process %d exiting\n"), num_child_procs );
+      pr_crit("child process %d exiting\n", num_child_procs);
       _exit(-1);
     }
     else return( err );
@@ -224,45 +247,148 @@ Load_Line( char *buff, FILE *pfile )
 
 /*------------------------------------------------------------------------*/
 
-/***  Memory allocation/freeing utils ***/
-static size_t cnt = 0; /* Total allocation */
-  void
-mem_alloc( void **ptr, size_t req, gchar *str )
+/* xnec2c uses mem_realloc() very often to resize buffers in the code.
+ * There are cases where uninitialized memory buffers can lead to incorrect behavior
+ * but unfortunately the libc realloc() call doesn't initialize the reallocated memory.
+ * 
+ * Since there is not a portable way to discover the amount of memory allocated by the
+ * previous realloc/malloc() call we cannot call memset() on the extended portion.
+ *
+ * To solve this mem_obj_t stores a couple sizes and a pointer:
+ *    1. size: the total allocated size requested by the caller
+ *             Note that the actual allocated size is greater by sizeof(mem_obj_t)
+ *    2. used: The amount actually used by the caller
+ *         a. If mem_realloc() is called with a smaller amount of memory requested
+ *            then it will shrink the `used` size without reallocating.  
+ *            
+ *         b. If mem_realloc() is called requesting more than `used` but less than `size`
+ *            then it is grown without reallocating.
+ *
+ *         c. If mem_realloc() is called requesting more than `size` then realloc() is
+ *            called and both `size` and `used` are updated.
+ *
+ *    3. ptr: a pointer to the memory used by the caller
+ *          
+ * A mem_obj_t object is allocated in excess of the structure size by the amount
+ * of memory requested by the caller.  When ptr_free or mem_realloc are called
+ * we use decrement by sizeof(mem_obj_t) to access the original
+ * structure pointer.
+ */
+typedef struct mem_obj_t
 {
-  gchar mesg[MESG_SIZE];
+	size_t size;
+	size_t used;
+	char **backtrace;
 
-  free_ptr( ptr );
-  *ptr = malloc( req );
-  cnt += req;
-  if( *ptr == NULL )
-  {
-    snprintf( mesg, sizeof(mesg),
-        _("Memory allocation denied %s\n"), str );
-    fprintf( stderr, "%s: Total memory request %lu\n", mesg, cnt );
-    Stop( mesg, ERR_STOP );
-  }
-  else
-	  memset(*ptr, 0, req);
+	void *ptr;
+} mem_obj_t;
 
+void mem_obj_dump(void *ptr)
+{
+	mem_obj_t *m = (mem_obj_t*)ptr;
+	m--;
+
+	pr_debug("mem_obj_t at %p:\n"
+		"  size: %lu\n"
+		"  used: %lu\n"
+		"  addr: %p\n", (void *)m, (unsigned long)m->size,  (unsigned long)m->used, m->ptr);
+
+
+}
+
+void mem_backtrace(void *ptr)
+{
+	mem_obj_t *m = (mem_obj_t *)ptr;
+	if (m == NULL)
+	{
+		print_backtrace("mem_backtrace(NULL)");
+
+		return;
+	}
+
+	m--;
+	pr_debug("mem_backtrace(%p):\n", ptr);
+	if (m->backtrace == NULL)
+		pr_debug("  m->backtrace is null, no backtrace data\n");
+	else
+		_print_backtrace(m->backtrace);
+
+}
+
+inline void mem_alloc( void **ptr, size_t req, gchar *str )
+{
+	mem_realloc(ptr, req, str);
 } /* End of mem_alloc() */
 
 /*------------------------------------------------------------------------*/
 
-  void
-mem_realloc( void **ptr, size_t req, gchar *str )
+void mem_realloc( void **ptr, size_t req, gchar *str )
 {
   gchar mesg[MESG_SIZE];
+  size_t prev_used;
 
-  *ptr = realloc( *ptr, req );
-  cnt += req;
-  if( *ptr == NULL )
+  // Get the original mem_obj_t object:
+  mem_obj_t *m = (mem_obj_t *)*ptr;
+
+  if (m == NULL)
+  {
+	  prev_used = 0;
+	  m = realloc( m, req+sizeof(mem_obj_t) );
+
+	  if (m != NULL)
+	  {
+		  m->size = req;
+		  m->used = req;
+		  m->backtrace = NULL;
+	  }
+  }
+  else
+  {
+	  // Adjust pointer to struct location:
+	  m--;
+
+	  if (m->backtrace != NULL)
+	  {
+		  free(m->backtrace);
+		  m->backtrace = NULL;
+	  }
+	  prev_used = m->used;
+
+	  if (req <= m->size)
+	  {
+		m->used = req;
+	  }
+	  else 
+	  {
+		m = realloc( m, req+sizeof(mem_obj_t) );
+
+		if (m != NULL)
+		{
+			m->size = req;
+			m->used = req;
+		}
+	  }
+  }
+
+  if( m == NULL )
   {
     snprintf( mesg, sizeof(mesg),
         _("Memory re-allocation denied %s\n"), str );
-    fprintf( stderr, "%s: Total memory request %lu\n", mesg, cnt );
+    pr_err("%s: Memory requested %lu\n", mesg, (unsigned long)req);
     Stop( mesg, ERR_STOP );
+	return;
   }
 
+  m->ptr = (void*)(m+1);
+  *ptr = m->ptr;
+
+  // Get a backtrace for the allocation.  Use mem_backtrace(ptr) to display it.
+  // Debug only if you need it, this is very slow:
+  //m->backtrace = _get_backtrace();
+
+  if (m->used > prev_used)
+	  memset(((uint8_t*)*ptr)+prev_used, 0x00, m->used - prev_used);
+
 } /* End of mem_realloc() */
 
 /*------------------------------------------------------------------------*/
@@ -271,7 +397,11 @@ mem_realloc( void **ptr, size_t req, gchar *str )
 free_ptr( void **ptr )
 {
   if( *ptr != NULL )
-    free( *ptr );
+  {
+	mem_obj_t *m = (mem_obj_t *)*ptr;
+	m--;
+    free( m );
+  }
   *ptr = NULL;
 
 } /* End of free_ptr() */
@@ -290,10 +420,12 @@ Open_File( FILE **fp, char *fname, const char *mode )
   if( (*fp = fopen(fname, mode)) == NULL )
   {
     char mesg[MESG_SIZE];
-    snprintf( mesg, sizeof(mesg), _("xnec2c[%d]: %s: Failed to open file\n"), getpid(), fname );
+    snprintf( mesg, sizeof(mesg), _("xnec2c[%d]: %s: Failed to open file: %s\n"), getpid(), fname, strerror(errno) );
     Stop( mesg, ERR_STOP );
     return( FALSE );
   }
+  else
+    pr_debug("%s: %s\n", mode, fname);
 
   return( TRUE );
 } /* Open_File() */
@@ -309,12 +441,10 @@ Close_File( FILE **fp )
 {
   if( *fp != NULL )
   {
-	  fsync(*fp);
+	  fsync(fileno(*fp));
 	  fclose(*fp);
-  }
-  else
 	  *fp = NULL;
-
+  }
 } /* Close_File() */
 
 /*------------------------------------------------------------------------*/
@@ -494,6 +624,24 @@ double Strtod( char *nptr, char **endptr )
 
 /*------------------------------------------------------------------*/
 
+/* str_append()
+ *
+ * Append a and b, store in dst.
+ *
+ * For example, this writes "hello world" to d:
+ *    char d[20];
+ *    str_append(d, "hello ", "world", 19);
+ */
+char *str_append(char *dst, char *a, char *b, size_t n)
+{
+  Strlcpy( dst, a, n );
+  Strlcat( dst, b, n );
+
+  return dst;
+}
+
+/*------------------------------------------------------------------*/
+
 /* Get_Dirname()
  *
  * Gets the directory name from a file path
@@ -520,10 +668,11 @@ Get_Dirname( char *fpath, char *dirname, int *fname_idx )
 
 typedef struct 
 {
-	GSourceFunc function;
+	GSourceOnceFunc function;
 	gpointer data;
 	GMutex lock;
 	int is_locked;
+	char **backtrace;
 } g_idle_add_data_t;
 
 
@@ -540,30 +689,57 @@ int _callback_g_idle_add_once(g_idle_add_data_t *cbdata)
 	{
 		// Wait for GTK
 		while (g_main_context_iteration(NULL, FALSE));
+		
+		if (cbdata->backtrace != NULL)
+		{
+			free(cbdata->backtrace);
+			cbdata->backtrace = NULL;
+		}
 
 		g_mutex_unlock(&cbdata->lock);
 	}
 	else
+	{
+		if (cbdata->backtrace != NULL)
+		{
+			cbdata->backtrace = NULL;
+			free(cbdata->backtrace);
+		}
+
 		free_ptr((void**)&cbdata);
+	}
+
 
 	return FALSE;
 }
 
-void _g_idle_add_once(GSourceFunc function, gpointer data, int lock)
+guint _g_idle_add_once(GSourceOnceFunc function, gpointer data, int lock)
 {
+	guint ret;
 	g_idle_add_data_t *cbdata = NULL;
+
 	mem_alloc((void**)&cbdata, sizeof(g_idle_add_data_t), __LOCATION__);
 
+	// Don't do a locked sync if -P is specified because GTK
+	// is already running synchronously.
+	if (rc_config.disable_pthread_freqloop)
+		lock = 0;
+
 	g_mutex_init(&cbdata->lock);
 
 	cbdata->function = function;
 	cbdata->data = data;
 	cbdata->is_locked = lock;
 
+	cbdata->backtrace = NULL;
+
+	// Debug async call backtraces if you need it, but be aware that _get_backtrace() is slow.
+	//cbdata->backtrace = _get_backtrace();
+
 	if (lock)
 		g_mutex_lock(&cbdata->lock);
 
-	g_idle_add((GSourceFunc)_callback_g_idle_add_once, cbdata);
+	ret = g_idle_add((GSourceFunc)_callback_g_idle_add_once, cbdata);
 
 	// Wait for the lock to release and free it.
 	if (lock)
@@ -572,18 +748,23 @@ void _g_idle_add_once(GSourceFunc function, gpointer data, int lock)
 		g_mutex_unlock(&cbdata->lock);
 		free_ptr((void**)&cbdata);
 	}
+
+	return ret;
 }
 
 // Call from any thread to queue a function to run once, do not wait for it to finish.
-void g_idle_add_once(GSourceFunc function, gpointer data)
+// This was implemented in glib 2.74 so use Gnome's version if available.
+#if GLIB_VERSION_CUR_STABLE < G_ENCODE_VERSION(2,74)
+guint g_idle_add_once(GSourceOnceFunc function, gpointer data)
 {
-	_g_idle_add_once(function, data, 0); // async
+	return _g_idle_add_once(function, data, 0); // async
 }
+#endif
 
 // Call from another thread to queue a function to run once, and wait for it to finish.
-void g_idle_add_once_sync(GSourceFunc function, gpointer data)
+guint g_idle_add_once_sync(GSourceOnceFunc function, gpointer data)
 {
-	_g_idle_add_once(function, data, 1); // sync
+	return _g_idle_add_once(function, data, 1); // sync
 }
 
 
@@ -594,10 +775,10 @@ void xnec2_widget_queue_draw(GtkWidget *w)
 	// Only redraw the rdpattern when FREQ_LOOP_DONE or it the window will flash grey:
 	if (w == rdpattern_drawingarea && isFlagSet(OPTIMIZER_OUTPUT) && isFlagSet(FREQ_LOOP_RUNNING))
 	{
-		printf("Optimizer loop incomplete, skipping radiation pattern redraw.\n");
+		pr_debug("Optimizer loop incomplete, skipping radiation pattern redraw.\n");
 	}
 	else
-		g_idle_add_once((GSourceFunc)gtk_widget_queue_draw, w);
+		g_idle_add_once((GSourceOnceFunc)gtk_widget_queue_draw, w);
 
 }
 /*------------------------------------------------------------------*/
@@ -606,21 +787,44 @@ void xnec2_widget_queue_draw(GtkWidget *w)
 /*
    Obtain a backtrace and print it to stdout. 
  */
-void print_backtrace(void)
+void _print_backtrace(char **strings)
 {
+	int i;
+	pr_debug("  Backtrace:\n");
+	for (i = 0; strings[i] != NULL; i++)
+		pr_debug("    %d. %s\n", i, strings[i]);
+}
+
+// Return an array of backtrace strings.  The value returned must be free()'ed.
+char **_get_backtrace()
+{
+#ifdef HAVE_BACKTRACE
 	void *array[10];
 	char **strings;
-	int size, i;
+	int size;
 
 	size = backtrace(array, 10);
 	strings = backtrace_symbols(array, size);
+
+	// This wastes an array entry, but allows _print_backtrace() to find
+	// the end of the list without realloc'ing space for a NULL:
+	strings[size-1] = NULL;
+
+	return strings;
+#else
+	return NULL;
+#endif
+}
+
+void print_backtrace(char *msg)
+{
+	if (msg != NULL)
+		pr_debug("%s:\n", msg);
+
+	char **strings = _get_backtrace();
 	if (strings != NULL)
 	{
-
-		printf("  Obtained %d stack frames.\n", size);
-		for (i = 0; i < size; i++)
-			printf("    %s\n", strings[i]);
+		_print_backtrace(strings);
+		free(strings);
 	}
-
-	free(strings);
 }
