@@ -30,6 +30,56 @@ static double tmp1, tmp2, tmp3, tmp4, tmp5, tmp6;
 
 /*-----------------------------------------------------------------------*/
 
+/* Set the calc_data.freq_step if it matches calc_data.freq_mhz.
+ * Redo radiation pattern for a new frequency.
+ *
+ * If it doesn't, return 0 so the caller can run New_Frequency() and
+ * use the extra buffer (in rad_pattern and other structures). */
+int set_freq_step()
+{
+	int fr, step;
+	double freq;
+
+	int prev_freq_step = calc_data.freq_step;
+
+	int idx = 0;
+	int found = 0;
+	for (fr = 0; !found && fr < calc_data.FR_cards && save.fstep[idx]; fr++)
+	{
+		freq = calc_data.freq_loop_data[fr].min_freq;
+		for (step = 0; !found && step < calc_data.freq_loop_data[fr].freq_steps && save.fstep[idx]; step++)
+		{
+			// if calc_data.freq_mhz =~ freq, +/- 1 Hz for rounding error:
+			if (calc_data.freq_mhz > freq - 1e-6 && calc_data.freq_mhz < freq + 1e-6)
+			{
+				calc_data.freq_step = idx;
+				found = 1;
+			}
+			else
+			{
+				if (calc_data.freq_loop_data[fr].ifreq == 1)
+					freq *= calc_data.freq_loop_data[fr].delta_freq;
+				else
+					freq += calc_data.freq_loop_data[fr].delta_freq;
+
+				idx++;
+			}
+
+		}
+	}
+
+	// If we didn't find the frequency, then use the "extra" frequency
+	// allocated as +1 at the end of all per-frequency data indexes:
+	if (!found)
+		calc_data.freq_step = calc_data.steps_total;
+
+	if (calc_data.freq_step != prev_freq_step)
+		SetFlag( DRAW_NEW_RDPAT );
+
+	// If we found the index, then no need to re-run New_Frequency because it is
+	// in the index.
+	return found;
+}
 /* Frequency_Scale_Geometry()
  *
  * Scales geometric parameters to frequency
@@ -132,12 +182,9 @@ Ground_Parameters( void )
         gnd.frati =( epsc - 1.0) / ( epsc + 1.0);
         if( cabs(( ggrid.epscf - epsc) / epsc) >= 1.0e-3 )
         {
-          fprintf( stderr,
-              _("xnec2c: Ground_Parameters(): error in ground parameters\n"
-              "complex dielectric constant from file: %12.5E%+12.5Ej\n"
-              "                            requested: %12.5E%+12.5Ej\n"),
-              creal(ggrid.epscf), cimag(ggrid.epscf),
-              creal(epsc), cimag(epsc) );
+          pr_err("complex dielectric constant from file: %12.5E%+12.5Ej, requested: %12.5E%+12.5Ej\n",
+                 creal(ggrid.epscf), cimag(ggrid.epscf),
+				 creal(epsc), cimag(epsc));
           Stop( _("Ground_Parameters():"
                 "Error in ground parameters"), ERR_STOP );
         }
@@ -268,12 +315,15 @@ Set_Network_Data( void )
   netcx.ntsol = 1;
 
   /* Save impedance data for normalization */
+  int fstep = calc_data.freq_step;
+  if (fstep < 0 || fstep > calc_data.steps_total)
+	return;
+
   if( ((calc_data.steps_total > 1) &&
-        isFlagSet(FREQ_LOOP_RUNNING)) || CHILD )
+        isFlagSet(FREQ_LOOP_RUNNING)) ||
+		CHILD ||
+		fstep == calc_data.steps_total)
   {
-    int fstep = calc_data.freq_step;
-    if (fstep < 0)
-        return;
 
     impedance_data.zreal[fstep] = (double)creal( netcx.zped);
     impedance_data.zimag[fstep] = (double)cimag( netcx.zped);
@@ -400,8 +450,7 @@ New_Frequency( void )
   save.last_freq = calc_data.freq_mhz;
 
   // Only show this if you manually change frequencies:
-  if (isFlagClear( FREQ_LOOP_RUNNING ))
-	clock_gettime(CLOCK_MONOTONIC, &start);
+  clock_gettime(CLOCK_MONOTONIC, &start);
 
   /* Frequency scaling of geometric parameters */
   Frequency_Scale_Geometry();
@@ -434,22 +483,12 @@ New_Frequency( void )
 
   g_mutex_unlock(&freq_data_lock);
 
-
-
-  // Only show this if you manually change frequencies:
-  if (isFlagClear( FREQ_LOOP_RUNNING ))
-  {
-	// Calculate elapsed time
-	clock_gettime(CLOCK_MONOTONIC, &end);
-
-	elapsed = (end.tv_sec + (double)end.tv_nsec/1e9) - (start.tv_sec + (double)start.tv_nsec/1e9);
-	printf("New_Frequency[%d]: %s: total time at %.2f MHz: %f seconds\n",
-		  getpid(),
-		  current_mathlib->name,
-		  calc_data.freq_mhz,
-		  elapsed
-	  );
-  }
+  // Calculate elapsed time
+  clock_gettime(CLOCK_MONOTONIC, &end);
+  
+  elapsed = (end.tv_sec + (double)end.tv_nsec/1e9) - (start.tv_sec + (double)start.tv_nsec/1e9);
+  pr_info("%.6f MHz: %f seconds. (%s)\n",
+			calc_data.freq_mhz, elapsed, current_mathlib->name);
 
 } /* New_Frequency()  */
 
@@ -487,9 +526,30 @@ int update_fmhz_save_spin_button_value(GtkSpinButton *w)
  *
  * Loops over frequency if calculations over a frequency range is
  * requested, dividing the job between child processes if forked
+ *
+ * This function does the following:
+
+	0. If FREQ_LOOP_INIT, set initial state
+	1. For each job:
+		a. Set "freq" to the right value from FR_cards for the next job to assign
+		b. If FREQ_LOOP_STOP, then cleanup and disable loop
+		c. Increment frequency, set save.freq[fstep] = (double)freq;
+		d. Find an available child and give it the fstep
+	5. Wait until a child has frequency data ready.
+		- For each available child:
+			a. Load the data via Get_Freq_Data
+			     into the fstep'th index that was handed to that particular child
+			b. Set save.fstep[child_fstep]=1 to indicate the data is ready
+			c. Clear the child's busy flag
+	6. Set calc_data.freq_step to the highest available step with data. 
+		This is an index value, not a count.
+	7. Trigger a redraw if still have busy children
+	8. If no more busy children:
+		Reset the saved frequency (green line) to what it was.
+	9. Save optimizer output if enabled
  */
-  gboolean
-Frequency_Loop( gpointer udata )
+
+gboolean Frequency_Loop( gpointer udata )
 {
   /* Value of frequency and step num in the loop */
   static double freq;
@@ -571,10 +631,12 @@ Frequency_Loop( gpointer udata )
    * execute only once, since only one instance is running */
   for( idx = 0; idx < calc_data.num_jobs; idx++ )
   {
-    /* Up frequency step count */
+    /* Up frequency step count. Note that this is initialized at -1 above so it
+	 * really does start the first loop with fstep==0 */
     fstep++;
 
-    /* If all steps of current FR card are processed, go to the next */
+    /* Set `freq` for use below
+     * If all steps of current FR card are processed, go to the next */
     if( fstep >= fsteps_total )
     {
       calc_data.FR_index++;
@@ -583,7 +645,9 @@ Frequency_Loop( gpointer udata )
         /* Add steps of new FR card range to total */
         fsteps_total += calc_data.freq_loop_data[calc_data.FR_index].freq_steps;
 
-        /* Update loop frequency from new FR card */
+        /* Update loop frequency from new FR card
+         * FIXME:  We shrink it here just so we cang row it down
+         * below where it is commented "Increment frequency" */
         freq = calc_data.freq_loop_data[calc_data.FR_index].min_freq;
         if( calc_data.freq_loop_data[calc_data.FR_index].ifreq == 1)
           freq /= calc_data.freq_loop_data[calc_data.FR_index].delta_freq;
@@ -592,19 +656,15 @@ Frequency_Loop( gpointer udata )
       }
       else
       {
-        SetFlag(FREQ_LOOP_STOP);
         calc_data.FR_index--;   /* keep it in range, avoid off-by-1 error */
       }
     }
 
-    /* Frequency loop is completed or was paused by user */
+    /* Frequency loop was paused by user */
     if( isFlagSet(FREQ_LOOP_STOP) )
     {
       /* Points to last buffer in rad_pattern filled by loop */
-      fstep--;
-
-      /* Last freq step that was processed by children */
-      calc_data.last_step = fstep;
+      fstep = calc_data.freq_step;
 
       /* Re-enable pausing of freq loop */
       ClearFlag( FREQ_LOOP_STOP );
@@ -615,17 +675,21 @@ Frequency_Loop( gpointer udata )
       break;
     } /* if( isFlagSet(FREQ_LOOP_STOP) ) */
 
-    /* Increment frequency */
+    /* Increment frequency:
+	 * ifreq is "IFRQ (I1) from the FR card specification:
+	 *    0 - linear stepping
+	 *    1 - multiplicative stepping */
     if( calc_data.freq_loop_data[calc_data.FR_index].ifreq == 1)
       freq *= calc_data.freq_loop_data[calc_data.FR_index].delta_freq;
     else
       freq += calc_data.freq_loop_data[calc_data.FR_index].delta_freq;
 
     /* Save frequencies for plotting */
-    save.freq[fstep] = (double)freq;
+    if (fstep < calc_data.steps_total)
+		save.freq[fstep] = (double)freq;
 
     /* Delegate calculations to child processes if forked */
-    if( FORKED )
+    if( FORKED && fstep < calc_data.steps_total )
     {
       /* Look for an idle process */
       for( job_num = 0; job_num < calc_data.num_jobs; job_num++ )
@@ -649,7 +713,7 @@ Frequency_Loop( gpointer udata )
           len = strlen( fork_commands[FRQDATA] );
           Write_Pipe( job_num, fork_commands[FRQDATA], (ssize_t)len, TRUE );
 
-          /* When it responds, give it next frequency */
+          /* Give it the next frequency */
           buff = (char *)&freq;
           len = sizeof( double );
           Write_Pipe( job_num, buff, (ssize_t)len, TRUE );
@@ -658,12 +722,20 @@ Frequency_Loop( gpointer udata )
       } /* for( job_num = 0; job_num < calc_data.num_jobs; job_num++ ) */
 
     } /* if( FORKED ) */
-    else /* Calculate freq dependent data (no fork) */
+    else if ( fstep < calc_data.steps_total ) /* Calculate freq dependent data (no fork) */
     {
+      g_mutex_lock(&freq_data_lock);
       calc_data.freq_mhz  = freq;
       calc_data.freq_step = fstep;
       calc_data.last_step = fstep;
+      g_mutex_unlock(&freq_data_lock);
+
       New_Frequency();
+
+      // Be sure to exit if this was the last iteration:
+      if (fstep >= calc_data.steps_total-1)
+             retval = 0;
+
       break;
     }
 
@@ -693,11 +765,13 @@ Frequency_Loop( gpointer udata )
       {
         if (errno == EINTR)
 			continue;
-        perror( "xnec2c: select()" );
+        perror( "select()" );
         _exit(0);
       }
 
       /* Check for finished child processes */
+      g_mutex_lock(&freq_data_lock);
+
       for( idx = 0; idx < num_child_procs; idx++ )
       {
         if( FD_ISSET(forked_proc_data[idx]->child2pnt_pipe[READ], &read_fds) )
@@ -705,6 +779,10 @@ Frequency_Loop( gpointer udata )
           /* Read data from finished child process */
           Get_Freq_Data( idx, forked_proc_data[idx]->fstep );
 
+          /* Clear "last-used-frequency" buffer, the local version of the data is
+           * no longer what New_Frequency() set it to: */
+          New_Frequency_Reset_Prev(); 
+
           /* Mark freq step in list of processed steps */
           save.fstep[forked_proc_data[idx]->fstep] = 1;
 
@@ -724,21 +802,49 @@ Frequency_Loop( gpointer udata )
         else break;
       }
 
+      if (calc_data.freq_step >= calc_data.steps_total-1)
+	  {
+		  /* Cancel idle callbacks on exit */
+		  retval = FALSE;
+		  if (num_busy_procs != 0)
+		  {
+			  BUG("Frequency_Loop is done with all steps but num_busy_procs=%d (calc_data.freq_step=%d calc_data.steps_total=%d)?\n",
+                              num_busy_procs, calc_data.freq_step,
+                              calc_data.steps_total);
+				for( job_num = 0; job_num < calc_data.num_jobs; job_num++ )
+				{
+					if (forked_proc_data[job_num]->busy)
+					{
+						pr_debug("  busy job[%d]: fstep=%d freq=%f\n",
+							job_num,
+							forked_proc_data[job_num]->fstep,
+							save.freq[forked_proc_data[job_num]->fstep]);
+					}
+				}
+		  }
+	  }
+
+      g_mutex_unlock(&freq_data_lock);
+
     } /* do. Loop terminated and busy children */
     while( !retval && num_busy_procs );
 
-  /* Return if freq step 0 not ready yet */
+  /* Return if freq step 0 not ready yet, no frequency data is populated yet. */
   if( calc_data.freq_step < 0 )
   {
 	  g_mutex_unlock(&global_lock);
       return( retval );
   }
 
-  /* Set frequency and step to global variables */
+  /* Set frequency and step to global variables
+   * FIXME: These should move within the &freq_data_lock section just above ^^ */
+  g_mutex_lock(&freq_data_lock);
   calc_data.last_step = calc_data.freq_step;
   calc_data.freq_mhz = (double)save.freq[calc_data.freq_step];
+  g_mutex_unlock(&freq_data_lock);
 
   SetFlag( FREQ_LOOP_READY );
+
   if (retval || num_busy_procs)
   {
 	/* Trigger a redraw of open drawingareas */
@@ -746,7 +852,7 @@ Frequency_Loop( gpointer udata )
 	if( isFlagSet(PLOT_ENABLED) )
 	{
 	  /* Display current frequency in plots entry */
-	  g_idle_add_once((GSourceFunc)update_freqplots_fmhz_entry, NULL);
+	  g_idle_add_once((GSourceOnceFunc)update_freqplots_fmhz_entry, NULL);
 
 	  if( isFlagClear(OPTIMIZER_OUTPUT) || freqplots_click_pending())
 	  {
@@ -755,11 +861,11 @@ Frequency_Loop( gpointer udata )
 	}
 
 	/* Set main window frequency spinbutton */
-	g_idle_add_once((GSourceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
+	g_idle_add_once((GSourceOnceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
 
 	/* Set Radiation pattern window frequency spinbutton */
 	if( isFlagSet(DRAW_ENABLED) )
-		  g_idle_add_once((GSourceFunc)update_freq_mhz_spin_button_value, rdpattern_frequency);
+		  g_idle_add_once((GSourceOnceFunc)update_freq_mhz_spin_button_value, rdpattern_frequency);
 
 	xnec2_widget_queue_draw( structure_drawingarea );
   }
@@ -771,8 +877,9 @@ Frequency_Loop( gpointer udata )
     SetFlag( FREQ_LOOP_DONE );
 
 	clock_gettime(CLOCK_MONOTONIC, &end);
-	printf("Frequency_Loop elapsed time: %f seconds\n", 
-		(end.tv_sec + (double)end.tv_nsec/1e9) - (start.tv_sec + (double)start.tv_nsec/1e9));
+	pr_notice("Frequency loop elapsed time: %f seconds. (%s)\n",
+                (end.tv_sec + (double)end.tv_nsec / 1e9) - (start.tv_sec + (double)start.tv_nsec / 1e9),
+				(FORKED ? get_mathlib_by_idx(rc_config.mathlib_batch_idx)->name : current_mathlib->name));
 
     /* After the loop is finished, re-set the saved frequency
      * that the user clicked on in the frequency plots window */
@@ -794,17 +901,17 @@ Frequency_Loop( gpointer udata )
 
       /* Set main window frequency spinbutton.
 	   * This will trigger New_Frequency() via Redo_Currents(). */
-      g_idle_add_once_sync((GSourceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
+      g_idle_add_once_sync((GSourceOnceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
 
       /* Set Radiation pattern window frequency spinbutton.
 	   * This will trigger New_Frequency() via Redo_Radiation_Pattern,. */
       if( isFlagSet(DRAW_ENABLED) )
-        g_idle_add_once_sync((GSourceFunc)update_fmhz_save_spin_button_value, rdpattern_frequency);
+        g_idle_add_once_sync((GSourceOnceFunc)update_fmhz_save_spin_button_value, rdpattern_frequency);
 
       if( isFlagSet(PLOT_ENABLED) )
       {
         SetFlag( PLOT_FREQ_LINE );
-        g_idle_add_once_sync((GSourceFunc)update_freqplots_fmhz_entry, NULL);
+        g_idle_add_once_sync((GSourceOnceFunc)update_freqplots_fmhz_entry, NULL);
       }
     }
 
@@ -814,11 +921,13 @@ Frequency_Loop( gpointer udata )
     if( isFlagSet(DRAW_ENABLED) )
       xnec2_widget_queue_draw( rdpattern_drawingarea );
 
-    /* Write out frequency loop data for
-     * the optimizer if SIGHUP received */
-    if( isFlagSet(OPTIMIZER_OUTPUT) )
+    /* Write out frequency loop data for the optimizer if optimization is
+     * active.  This is a sync call, so only call this if we have files flagged
+     * for writing (opt_have_files_to_save) based on the checkboxes in
+     * File->Optimization Settings */
+    if( (rc_config.batch_mode || isFlagSet(OPTIMIZER_OUTPUT)) && opt_have_files_to_save() )
     {
-      g_idle_add_once_sync((GSourceFunc)Write_Optimizer_Data, NULL);
+      g_idle_add_once_sync((GSourceOnceFunc)Write_Optimizer_Data, NULL);
     }
   } // if( !retval && !num_busy_procs )
 
@@ -832,9 +941,22 @@ Frequency_Loop( gpointer udata )
 
 void *Frequency_Loop_Thread(void *p)
 {
+	// Don't draw the green line if in batch mode
+	if (rc_config.batch_mode)
+		calc_data.fmhz_save = 0.0;
+
+	// Run the loop
 	while (isFlagSet(FREQ_LOOP_RUNNING) && Frequency_Loop(NULL));
 
 	ClearFlag(FREQ_LOOP_RUNNING);
+
+	// Exit if in batch mode
+	if (rc_config.batch_mode)
+	{
+		g_idle_add_once_sync((GSourceOnceFunc)Gtk_Quit, NULL);
+		return NULL;
+	}
+
 	SetFlag(DRAW_NEW_RDPAT);
 
 	/*
@@ -852,14 +974,14 @@ void *Frequency_Loop_Thread(void *p)
 	   Re-draw drawing areas at end of loop 
 	 */
 	if (isFlagSet(PLOT_ENABLED))
-		g_idle_add_once_sync((GSourceFunc) gtk_widget_queue_draw, freqplots_drawingarea);
+		g_idle_add_once_sync((GSourceOnceFunc) gtk_widget_queue_draw, freqplots_drawingarea);
 
 	if (isFlagSet(DRAW_ENABLED))
 	{
-		g_idle_add_once_sync((GSourceFunc) update_freq_mhz_spin_button_value, rdpattern_frequency);
-		g_idle_add_once_sync((GSourceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
-		g_idle_add_once_sync(Redo_Currents, NULL);
-		g_idle_add_once_sync((GSourceFunc) gtk_widget_queue_draw, rdpattern_drawingarea);
+		g_idle_add_once_sync((GSourceOnceFunc)update_freq_mhz_spin_button_value, rdpattern_frequency);
+		g_idle_add_once_sync((GSourceOnceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
+		g_idle_add_once_sync((GSourceOnceFunc)Redo_Currents, NULL);
+		g_idle_add_once_sync((GSourceOnceFunc)gtk_widget_queue_draw, rdpattern_drawingarea);
 	}
 
 	return NULL;
@@ -883,15 +1005,23 @@ Start_Frequency_Loop( void )
     retval = TRUE;
     SetFlag( FREQ_LOOP_INIT );
     SetFlag( FREQ_LOOP_RUNNING );
-    mem_alloc((void**)&pth_freq_loop, sizeof(pthread_t), __LOCATION__);
-    int ret = pthread_create( pth_freq_loop, NULL, Frequency_Loop_Thread, NULL );
-    if( ret != 0 )
-    {
-        free_ptr((void**)&pth_freq_loop);
-        fprintf( stderr, "xnec2c: failed to start Frequency_Loop_Thread\n" );
-        perror( "xnec2c: pthread_create()" );
-        exit( -1 );
-    }
+
+	if (!rc_config.disable_pthread_freqloop)
+	{
+		mem_alloc((void**)&pth_freq_loop, sizeof(pthread_t), __LOCATION__);
+		int ret = pthread_create( pth_freq_loop, NULL, Frequency_Loop_Thread, NULL );
+		if( ret != 0 )
+		{
+			free_ptr((void**)&pth_freq_loop);
+			pr_crit("failed to start Frequency_Loop_Thread\n");
+			perror( "pthread_create()" );
+			exit( -1 );
+		}
+	}
+	else
+	{
+		floop_tag = g_idle_add( Frequency_Loop, NULL );
+	}
     return( TRUE );
   }
   else return( FALSE );
@@ -910,11 +1040,19 @@ Stop_Frequency_Loop( void )
   // Clearing this flag will cause the Frequency_Loop pthread to exit when it is done:
   ClearFlag( FREQ_LOOP_RUNNING );
 
-  // Wait for the thread to exit:
-  if (pth_freq_loop != NULL)
+  if (!rc_config.disable_pthread_freqloop)
   {
-	  pthread_join(*pth_freq_loop, NULL);
-	  free_ptr((void**)&pth_freq_loop);
+	  // Wait for the thread to exit:
+	  if (pth_freq_loop != NULL)
+	  {
+		  pthread_join(*pth_freq_loop, NULL);
+		  free_ptr((void**)&pth_freq_loop);
+	  }
+  }
+  else if( floop_tag > 0 )
+  {
+	g_source_remove( floop_tag );
+	floop_tag = 0;
   }
 } /* Stop_Frequency_Loop() */
 
